//	**************************************************************************************************
//	Name	:	filecopy.hxx
//			:	File Copy
//	Author	:	I. Nakagawa
//	Create	:	2015/01/15
//	Modify	:	2019/02/04
//	Modify	:	2020/01/08
//	**************************************************************************************************

#pragma		once

#ifdef		_WIN32
#include	<Windows.h>
#endif
#ifdef		__GNUC__
#include	<cstdlib>
#endif

//#include	"DefNonMS.hxx"
#include	"existff.hxx"
#include	"createff.hxx"
#include	"quotm.hxx"

////
//	*******************************************************************************
//	File Copy
//	Create	:	2015/01/15
//	Modify	:	2020/01/08	cp -p
//	*******************************************************************************
inline	bool	File_Copy	(LPCTSTR src,LPCTSTR dst,const bool bFailIfExists=false)
{
	{
		if		(::File_IsNothing	(src))		{	return	false ;		}
		if (bFailIfExists) {
			if	(::File_IsExist		(dst))		{	return	false ;		}
			}
		}
	{
		tstring	dstFolder = ::Path_GetDir(dst) ;
		if (::Folder_IsNothing	(dstFolder)) {
			if (!::Folder_Create(dstFolder))	{	return	false ;		}
			}
		}
	#ifdef	_MSC_VER
	{
		BOOL	result = FALSE ;
		if (bFailIfExists)	{	result = ::CopyFile(src,dst,TRUE) ;		}
		else				{	result = ::CopyFile(src,dst,FALSE) ;	}
		if (result)			{	return	true ;	}
		}
	#endif
	#ifdef	__GNUC__
	{
		tstring	src_	=	::QuotM_Add(src) ;
		tstring	dst_	=	::QuotM_Add(dst) ;
	//	tstring	cp_cmd ;	cp_cmd = _T("cp ")    + src_ + _T(" ") + dst_ ;
		tstring	cp_cmd ;	cp_cmd = _T("cp -p ") + src_ + _T(" ") + dst_ ;
		int		result	=	system(cp_cmd.c_str()) ;
		if (result == 0)	{	return	true ;	}
		}
	#endif
	return	false ;
	}

////
//	*******************************************************************************
//	File Move
//	Create	:	2015/01/15
//	*******************************************************************************
inline	bool	File_Move	(LPCTSTR src,LPCTSTR dst,const bool bFailIfExists=false)
{
	{
		if		(::File_IsNothing	(src))		{	return	false ;		}
		if (bFailIfExists) {
			if	(::File_IsExist		(dst))		{	return	false ;		}
			}
		}
	{
		tstring	dstFolder = ::Path_GetDir(dst) ;
		if (::Folder_IsNothing	(dstFolder)) {
			if (!::Folder_Create(dstFolder))	{	return	false ;		}
			}
		}
	#ifdef	_MSC_VER
	{
		BOOL	result = ::MoveFile(src,dst) ;
		if (result)								{	return	true ;		}
		}
	#endif
	if (!::File_Copy(src,dst,bFailIfExists))	{	return	false ;		}
	int	result = _tremove(src) ;
	if (result == 0)							{	return	true ;		}
	return	false ;
	}

////
//	*******************************************************************************
//	file copy , move		tstring
//	Create	:	2019/02/04
//	*******************************************************************************
inline	bool	File_Copy	(c_tstring& src,c_tstring& dst,const bool bFailIfExists=false)
	{	return	::File_Copy(src.c_str(),dst.c_str(),bFailIfExists) ;	}
inline	bool	File_Move	(c_tstring& src,c_tstring& dst,const bool bFailIfExists=false)
	{	return	::File_Move(src.c_str(),dst.c_str(),bFailIfExists) ;	}
