//	*******************************************************************************
//	Name	:	S_Brwose.hxx
//			:	SHBrowseForFolder
//	Author	:	I. Nakagawa
//	Create	:	2021/06/11
//	Modify	:	2021/07/02
//	*******************************************************************************

#pragma		once

#include	<ShlObj.h>
#include	"SH_IDL.hxx"
#include	"filestat.hxx"

//	(VS8)\VC\PlatformSDK\Include\ShlObj.h
#ifndef	BIF_NEWDIALOGSTYLE
#define BIF_NEWDIALOGSTYLE		0x0040	// Use the new dialog layout with the ability to resize
#define BIF_USENEWUI			(BIF_NEWDIALOGSTYLE | BIF_EDITBOX)
#define BIF_BROWSEINCLUDEURLS	0x0080	// Allow URLs to be displayed or entered. (Requires BIF_USENEWUI)
#define BIF_UAHINT				0x0100	// Add a UA hint to the dialog, in place of the edit box. May not be combined with BIF_EDITBOX
#define BIF_NONEWFOLDERBUTTON	0x0200	// Do not add the "New Folder" button to the dialog.  Only applicable with BIF_NEWDIALOGSTYLE.
#define BIF_NOTRANSLATETARGETS	0x0400	// don't traverse target as shortcut
#endif

////
//	*******************************************************************************
//	BrowseForFolder callback
//	Create	:	2021/06/11
//	*******************************************************************************
inline	int		CALLBACK	SH_BrowseCallback	(HWND hwnd,UINT uMsg,LPARAM lParam,LPARAM lpData)
{
	switch	(uMsg) {
		case	BFFM_INITIALIZED :
			::SendMessage(hwnd,BFFM_SETSELECTION,1,(LPARAM)(LPCTSTR)lpData) ;
			return	1 ;
			break ;
		default :
			return	0 ;
			break ;
		}
	return	0 ;
	}

////
//	*******************************************************************************
//	browse for folder
//	Create	:	2021/06/11
//	Create	:	2021/07/02
//	*******************************************************************************
inline	tstring	Browse_Folder	(HWND hwnd,LPCTSTR init_fold_=_T(""),LPCTSTR title=_T(""))
{
	tstring			init_fold = init_fold_ ;
	if (!::File_IsDirectory(init_fold)) {
			init_fold = ::Path_GetDir(init_fold) ;
		}
	BROWSEINFO		 bi	=	{	0	} ;
	bi.hwndOwner	 =	hwnd ;
	bi.ulFlags		|=	BIF_RETURNONLYFSDIRS ;
	bi.ulFlags		|=	BIF_NEWDIALOGSTYLE ;
	bi.ulFlags		|=	BIF_NONEWFOLDERBUTTON ;
	bi.lpfn			 =	SH_BrowseCallback ;
	bi.lParam		 =	(LPARAM)init_fold.c_str() ;
	bi.lpszTitle	 =	title ;
	LPITEMIDLIST	pidl = ::SHBrowseForFolder(&bi) ;
	if (pidl == NULL)	{	return	tstring() ;		}
	tstring	path = ::SH_GetPathFromIDList(pidl) ;
	::CoTaskMemFree(pidl) ;
	return	path ;
	}

