//	*******************************************************************************
//	Name	:	enumfile.hxx
//			:	Enumerate Files
//	Author	:	I. Nakagawa
//	Create	:	2014/12/17
//	Modify	:	2023/07/11		add EnumFilesTree(c_v_tstring& ...)
//	Modify	:	2024/09/09		LPCTSTR		_MFC_VER
//	*******************************************************************************

#pragma		once

#ifdef		_MSC_VER
#pragma		warning	(disable	:	4786	)
#endif
#include	<utility>

#ifdef		_MSC_VER
#include	<Windows.h>
#endif
#ifdef		_MFC_VER
#include	"i_FileF.hxx"
#endif
#ifdef		__GNUC__
#include	<dirent.h>
#endif

#include	"existff.hxx"
#include	"filestat.hxx"
#include	"filepath.hxx"

#ifdef		__GNUC__
#else
#ifdef		MessageBar
#else
#pragma		message	("enumfile.hxx MessageBar  No Support")
#endif
#endif

#if			(_MSC_VER == 1900)
#include	"FileAttr.hxx"
#endif

////
#ifdef	__GNUC__
//	*******************************************************************************
//	Enumerate Files in a Folder		GNUC
//	Create	:	2014/12/19
//	Modify	:	2014/12/22
//	Modify	:	2017/05/26		bug opendir EXC_BAD_ACCESS
//	*******************************************************************************
inline	v_tstring	EnumFiles_GNUC	(c_tstring& path_,const bool skipDot=true)
{
	tstring	path = path_ ;
	if (!::File_IsExist(path))				{	path = ::Path_GetDir(path) ;	}
	if (!::File_IsDirectory(path.c_str()))	{	path = ::Path_GetDir(path) ;	}
	v_tstring	foundFiles ;
	{
		DIR*	dp = ::opendir(path.c_str()) ;
		struct	dirent*	dent = NULL ;
		do	{
			dent = readdir(dp) ;
			if (dent != NULL)	{
				tstring	fileName = dent->d_name ;
				if (skipDot) {
					if (fileName == _T("."))	{	continue ;	}
					if (fileName == _T(".."))	{	continue ;	}
					}
				if (dent->d_type & DT_DIR) {
					fileName = ::Path_AddLastSP(fileName) ;
					}
				foundFiles.push_back(::Path_AddLastSP(path)+fileName) ;
				}
			}	while (dent != NULL) ;
		::closedir(dp) ;
		}
	return	foundFiles ;
	}

//	*******************************************************************************
//	Is Exist File in a Folder		GNUC
//	Create	:	2014/12/19
//	Modify	:	2017/05/26
//	*******************************************************************************
inline	bool	Folder_IsEmpty_GNUC	(c_tstring& path_)
{
	tstring	path = path_ ;
	if (!::File_IsExist(path))				{	path = ::Path_GetDir(path) ;	}
	if (!::File_IsDirectory(path.c_str()))	{	path = ::Path_GetDir(path) ;	}
	bool	isEmpty = true ;
	{
		DIR*	dp = ::opendir(path.c_str()) ;
		struct	dirent*	dent = NULL ;
		do	{
			dent = readdir(dp) ;
			if (dent != NULL)	{
				tstring	fileName = dent->d_name ;
				if (fileName == _T("."))	{	continue ;	}
				if (fileName == _T(".."))	{	continue ;	}
				isEmpty = false ;
				break ;
				}
			}	while (dent != NULL) ;
		::closedir(dp) ;
		}
	return	isEmpty ;
	}

#endif

////
#ifdef	_MSC_VER
//	*******************************************************************************
//	Enumerate Files in a Folder		MSC
//	Create	:	2014/12/19
//	Modify	:	2014/12/22
//	*******************************************************************************
inline	v_tstring	EnumFiles_MSC	(c_tstring& path,const bool skipDot=true)
{
	v_tstring	foundFiles ;
	{
		tstring	fffPath = ::Path_AddLastSP(path)+_T("*.*") ;
		WIN32_FIND_DATA	fd ;
		memset(&fd,0,sizeof(WIN32_FIND_DATA)) ;
		HANDLE	hFind = ::FindFirstFile(fffPath.c_str(),&fd) ;
		if (hFind != INVALID_HANDLE_VALUE) {
			while	(TRUE)	{
				tstring	fileName = fd.cFileName ;
				if (skipDot) {
					if (fileName == _T("."))	{	fileName = _T("")	;	}
					if (fileName == _T(".."))	{	fileName = _T("")	;	}
					}
				if (!fileName.empty()) {
					if (fd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) {
						fileName = ::Path_AddLastSP(fileName) ;
						}
					foundFiles.push_back(::Path_AddLastSP(path)+fileName) ;
					}
				if (!::FindNextFile(hFind,&fd)) {
					break ;
					}
				}
			::FindClose(hFind) ;
			}
		}
	return	foundFiles ;
	}

//	*******************************************************************************
//	Is Exist File in a Folder		MSC
//	Create	:	2014/12/19
//	*******************************************************************************
inline	bool	Folder_IsEmpty_MSC	(c_tstring& path)
{
	bool	isEmpty = true ;
	{
		tstring	fffPath = ::Path_AddLastSP(path)+_T("*.*") ;
		WIN32_FIND_DATA	fd ;
		memset(&fd,0,sizeof(WIN32_FIND_DATA)) ;
		HANDLE	hFind = ::FindFirstFile(fffPath.c_str(),&fd) ;
		if (hFind != INVALID_HANDLE_VALUE) {
			while	(TRUE)	{
				tstring	fileName = fd.cFileName ;
				if (fileName == _T("."))	{	fileName = _T("")	;	}
				if (fileName == _T(".."))	{	fileName = _T("")	;	}
				if (!fileName.empty()) {
					isEmpty = false ;
					break ;
					}
				if (!::FindNextFile(hFind,&fd)) {
					break ;
					}
				}
			::FindClose(hFind) ;
			}
		}
	return	isEmpty ;
	}

#endif

////
#ifdef	_MFC_VER
//	*******************************************************************************
//	Enumerate Files in a Folder		MFC
//	Create	:	2014/12/19
//	Modify	:	2014/12/22
//	Modify	:	2024/09/09		LPCTSTR
//	*******************************************************************************
inline	v_tstring	EnumFiles_MFC	(c_tstring& path,const bool skipDot=true)
{
	v_tstring	foundFiles ;
	{
		iFileFind	ff ;
		BOOL		isWorking = ff.FindFile((::Path_AddLastSP(path)+_T("*.*")).c_str()) ;
		while	(isWorking)	{
					isWorking = ff.FindNextFile() ;
			if (skipDot && ff.IsDots())				{	continue ;	}
			tstring	fileName = LPCTSTR(ff.GetFileName()) ;
			if (ff.IsDirectory()) {
				fileName = ::Path_AddLastSP(fileName) ;
				}
			foundFiles.push_back(::Path_AddLastSP(path)+fileName) ;
			}
		}
	return	foundFiles ;
	}

//	*******************************************************************************
//	Is Exist File in a Folder		MFC
//	Create	:	2014/12/19
//	*******************************************************************************
inline	bool	Folder_IsEmpty_MFC	(c_tstring& path)
{
	bool	isEmpty = true ;
	{
		iFileFind	ff ;
		BOOL		isWorking = ff.FindFile((::Path_AddLastSP(path)+_T("*.*")).c_str()) ;
		while	(isWorking)	{
					isWorking = ff.FindNextFile() ;
			if (ff.IsDots())						{	continue ;	}
			isEmpty = false ;
			break ;
			}
		}
	return	isEmpty ;
	}

#endif

////
//	*******************************************************************************
//	Enumerate Files in a Folder		folder is Path_SP termination
//	Create	:	2014/12/18
//	Modify	:	2017/03/14	for VC 14
//	*******************************************************************************
inline	v_tstring	EnumFiles	(LPCTSTR path_,const bool skipDot=true)
{
	tstring	path = ::Path_DelLastSP(path_) ;
	if (!::File_IsDirectory(path.c_str()))	{
		#if	(_MSC_VER == 1900)
		{
			if (!::FA_Is_Directory(path)) {
				path = ::Path_GetDir(path) ;
				}
			}
		#else
		{
			path = ::Path_GetDir(path) ;
			}
		#endif
		}
	v_tstring	foundFiles ;
	#if		defined	__GNUC__
		foundFiles = ::EnumFiles_GNUC	(path,skipDot) ;
	#elif	defined	_MFC_VER
		foundFiles = ::EnumFiles_MFC	(path,skipDot) ;
	#elif	defined	_MSC_VER
		foundFiles = ::EnumFiles_MSC	(path,skipDot) ;
	#endif
	return	foundFiles ;
	}

////
//	*******************************************************************************
//	Is Exist File in a Folder
//	Create	:	2014/12/19
//	*******************************************************************************
inline	bool	Folder_IsEmpty	(LPCTSTR path_)
{
	tstring	path = ::Path_DelLastSP(path_) ;
	if (!File_IsDirectory(path.c_str()))	{
		path = ::Path_GetDir(path) ;
		}
	bool	isEmpty = true ;
	#if		defined	__GNUC__
		isEmpty = ::Folder_IsEmpty_GNUC	(path) ;
	#elif	defined	_MFC_VER
		isEmpty = ::Folder_IsEmpty_MFC	(path) ;
	#elif	defined	_MSC_VER
		isEmpty = ::Folder_IsEmpty_MSC	(path) ;
	#endif
	return	isEmpty ;
	}

////
//	*******************************************************************************
//	Do Extract Files
//	Create	:	2014/12/22
//	*******************************************************************************
inline	bool	ExtractFiles_match	(c_tstring& filePath,c_tstring& wildName_)
{
	bool	match = false ;
	tstring	fileName = ::Path_GetName(filePath) ;
	tstring	wildName = wildName_ ;
			fileName = ::String_ToLower(fileName) ;
			wildName = ::String_ToLower(wildName) ;
	if		(wildName.empty())				{	match = true ;	}
	else if (wildName == _T("*"))			{	match = true ;	}
	else if (wildName == _T("*.*"))	{
		tstring::size_type	dotPos = fileName.find(_T('.')) ;
		if (dotPos != tstring::npos)		{	match = true ;	}
		}
	else {
		v_tstring	patterns = ::String_Split(wildName,_T('*')) ;
					patterns = ::SkipEmpty(patterns) ;
		tstring		checkName= fileName ;
		bool		found = true ;
		for (size_t index=0 ; index<patterns.size() ; index++) {
			tstring	pattern = patterns[index] ;
			if (pattern.empty())			{	continue ;	}
			tstring::size_type	patPos = checkName.find(pattern) ;
			if (patPos == tstring::npos)	{	found = false ;	break ;	}
			checkName = checkName.substr(patPos+pattern.size()) ;
			}
		if (found) {
			bool	patt_1st = (wildName[0] 				!= _T('*')) ;		//	1st is pattern ?
			bool	pattLast = (wildName[wildName.size()-1]	!= _T('*')) ;
			bool	match_1st = true ;
			bool	matchLast = true ;
			if (patt_1st) {
				tstring	ptrn_1st = patterns[0] ;
				tstring	name_1st = fileName.substr(0,ptrn_1st.size()) ;
				if (ptrn_1st != name_1st)	{	match_1st = false ;		}
				}
			if (pattLast) {
				tstring	ptrnLast = patterns[patterns.size()-1] ;
				tstring	nameLast = fileName.substr(fileName.size()-ptrnLast.size()) ;
				if (ptrnLast != nameLast)	{	matchLast = false ;		}
				}
			if (match_1st && matchLast)		{	match = true ;	}
			}
		}
	return	match ;
	}

////
//	*******************************************************************************
//	Do Extract Files
//	Create	:	2014/12/22
//	*******************************************************************************
inline	v_tstring	ExtractFiles	(c_v_tstring& foundFiles,c_tstring& wildName)
{
	v_tstring	extraFiles ;
	#ifdef		MessageBar
	MessageBar	bar(_T("Extract files ..."),foundFiles.size()) ;
	#endif
	for (size_t index=0 ; index<foundFiles.size() ; index++) {
	#ifdef		MessageBar
		bar.SetBarInc() ;
	#endif
		tstring	file = foundFiles[index] ;
		if (::Path_LastIsSP(file))	{	continue ;	}	//	skip directory
		if (::ExtractFiles_match(file,wildName)) {
			extraFiles.push_back(file) ;
			}
		}
	return	extraFiles ;
	}

////
//	*******************************************************************************
//	Do Extract Folder		input folder is Path_SP termination
//	Create	:	2014/12/22
//	*******************************************************************************
inline	v_tstring	ExtractFolders	(c_v_tstring& foundFiles)
{
	v_tstring	foundFolds ;
	for (size_t index=0 ; index<foundFiles.size() ; index++) {
		tstring	file = foundFiles[index] ;
		if (::Path_LastIsSP(file)) {
			foundFolds.push_back(file) ;
			}
		}
	return	foundFolds ;
	}

////
//	*******************************************************************************
//	Do Extract Folder		use stat function to check
//	Create	:	2014/12/19
//	*******************************************************************************
inline	v_tstring	ExtractFolders_stat	(c_v_tstring& foundFiles)
{
	v_tstring	foundFolds ;
	for (size_t index=0 ; index<foundFiles.size() ; index++) {
		tstring	file = foundFiles[index] ;
		if (::File_IsDirectory(file.c_str())) {
			foundFolds.push_back(file) ;
			}
		}
	return	foundFolds ;
	}


////
//	*******************************************************************************
//	Enumerate Files		one folder	specified wild card
//	Create	:	2015/01/15
//	Modify	:	2021/07/26	add 2 functions
//	*******************************************************************************
inline	v_tstring	EnumFiles	(LPCTSTR path,c_tstring& wildName)
{
	v_tstring	foundFiles = ::EnumFiles		(path) ;
	v_tstring	extraFiles = ::ExtractFiles	(foundFiles,wildName) ;
	return	extraFiles ;
	}

inline	v_tstring	EnumFiles	(c_tstring& path,c_tstring& wildName)	{	return	::EnumFiles(path.c_str(),        wildName) ;	}
inline	v_tstring	EnumFiles	(c_tstring& path,LPCTSTR    wildName)	{	return	::EnumFiles(path.c_str(),tstring(wildName)) ;	}
inline	v_tstring	EnumFiles	(LPCTSTR    path,LPCTSTR    wildName)	{	return	::EnumFiles(path,        tstring(wildName)) ;	}

////
//	*******************************************************************************
//	Enumerate Files		one folder	specified wild card		n types
//	Create	:	2020/09/02
//	*******************************************************************************
inline	v_tstring	EnumFiles	(LPCTSTR path,c_v_tstring& wilds)
{
	v_tstring	found_files = ::EnumFiles(path) ;
				found_files = ::Sort(found_files) ;
	v_tstring	extraFiles ;
	for (size_t index=0 ; index<wilds.size() ; index++) {
		tstring	wildName = wilds[index] ;
		v_tstring	extra= ::ExtractFiles(found_files,wildName) ;
		extraFiles.insert(extraFiles.end(),extra.begin(),extra.end()) ;
		}
	return	extraFiles ;
	}

////
//	*******************************************************************************
//	Enumerate Folder
//	Create	:	2014/12/19
//	*******************************************************************************
inline	v_tstring	EnumFolders	(LPCTSTR path)
{
	v_tstring	foundFiles = ::EnumFiles(path) ;
	v_tstring	foundFolds = ::ExtractFolders(foundFiles) ;
	return	foundFolds ;
	}

////
//	*******************************************************************************
//	Enumerate Folder Tree
//	Create	:	2014/12/19
//	Modify	:	2021/02/15	bug	UNICODE folder
//	*******************************************************************************
inline	v_tstring	EnumFoldersTree	(LPCTSTR path)
{
	v_tstring	foundFolds = ::EnumFolders(path) ;
	#ifdef		MessageBar
		tstring	fold_abb = ::Path_GetAbbreviate(path,20) ;
		MessageBar	bar((fold_abb + _T(" Enumerate folders ...")).c_str(),foundFolds.size()) ;
	#endif
	for (size_t index=0 ; index<foundFolds.size() ; index++) {
		#ifdef		MessageBar
			bar.SetBarMax(foundFolds.size()) ;
			bar.SetBarInc() ;
		#endif
		tstring		subFold = foundFolds[index] ;
		if (::File_IsNothing(::Path_DelLastSP(subFold)))	{		continue  ;		}
		v_tstring	chFolds = ::EnumFolders(subFold.c_str()) ;
		foundFolds.insert(foundFolds.end(),chFolds.begin(),chFolds.end()) ;
		}
	return	foundFolds ;
	}

////
//	*******************************************************************************
//	Enumerate Files Tree	folder is Path_SP termination
//	Create	:	2014/12/22
//	*******************************************************************************
inline	v_tstring	EnumFilesTree1	(LPCTSTR path)
{
	v_tstring	foundFiles = ::EnumFiles(path) ;
	v_tstring	foundFolds = ::ExtractFolders(foundFiles) ;
	#ifdef		MessageBar
	MessageBar	bar(_T("Enumerate files ..."),foundFolds.size()) ;
	#ifdef	_OPENMP
				bar.SetMessage(_T("Enumerate files omp ...")) ;
	#endif
	#endif
	for (size_t index=0 ; index<foundFolds.size() ; index++) {
		tstring		subFold = foundFolds[index] ;
		v_tstring	chFiles = ::EnumFiles(subFold.c_str()) ;
		v_tstring	chFolds = ::ExtractFolders(chFiles) ;
		foundFolds.insert(foundFolds.end(),chFolds.begin(),chFolds.end()) ;
		foundFiles.insert(foundFiles.end(),chFiles.begin(),chFiles.end()) ;
	#ifdef		MessageBar
		bar.SetBarMax(foundFolds.size()) ;
		bar.SetBarInc() ;
	#endif
		}
	return	foundFiles ;
	}

////
//	*******************************************************************************
//	Enumerate Files Tree	OpenMP
//	Create	:	2015/01/05
//	*******************************************************************************
inline	v_tstring	EnumFilesTree_MT	(LPCTSTR path)
{
	v_tstring	foundFiles = ::EnumFiles(path) ;
	v_tstring	foundFolds = ::ExtractFolders(foundFiles) ;
	#ifdef		MessageBar
	MessageBar	bar(_T("Enumerate files ..."),foundFolds.size()) ;
	#ifdef	_OPENMP
				bar.SetMessage(_T("Enumerate files omp ...")) ;
		#pragma	omp	parallel for
	#endif
	#endif
	for (int index=0 ; index<int(foundFolds.size()) ; index++) {
		tstring		subFold = foundFolds[index] ;
		v_tstring	chFiles = ::EnumFilesTree1(subFold.c_str()) ;
		#ifdef	_OPENMP
			#pragma	omp	critical	(EnumFilesTree_MT)
		#endif
		{
			foundFiles.insert(foundFiles.end(),chFiles.begin(),chFiles.end()) ;
	#ifdef		MessageBar
			bar.SetBarInc() ;
	#endif
			}
		}
	return	foundFiles ;
	}

////
//	*******************************************************************************
//	Enumerate Files Tree	folder is Path_SP termination
//	Create	:	2014/12/22
//	Modify	:	2015/01/05
//	*******************************************************************************
inline	v_tstring	EnumFilesTree	(LPCTSTR path)
{
	#ifdef	_OPENMP
	v_tstring	foundFiles = ::EnumFilesTree_MT	(path) ;
	#else
	v_tstring	foundFiles = ::EnumFilesTree1	(path) ;
	#endif
	return		foundFiles ;
	}
/*
inline	v_tstring	EnumFilesTree_Old	(LPCTSTR path)
{
	v_tstring	foundFiles = ::EnumFiles(path) ;
	v_tstring	foundFolds = ::ExtractFolders(foundFiles) ;
	#ifdef		MessageBar
	MessageBar	bar(_T("Enumerate files ..."),foundFolds.size()) ;
	#ifdef	_OPENMP
				bar.SetMessage(_T("Enumerate files omp ...")) ;
	#endif
	#endif
	for (size_t index=0 ; index<foundFolds.size() ; index++) {
		tstring		subFold = foundFolds[index] ;
		v_tstring	chFiles = ::EnumFiles(subFold.c_str()) ;
		v_tstring	chFolds = ::ExtractFolders(chFiles) ;
		foundFolds.insert(foundFolds.end(),chFolds.begin(),chFolds.end()) ;
		foundFiles.insert(foundFiles.end(),chFiles.begin(),chFiles.end()) ;
	#ifdef		MessageBar
		bar.SetBarMax(foundFolds.size()) ;
		bar.SetBarInc() ;
	#endif
		}
	return	foundFiles ;
	}
inline	v_tstring	EnumFilesTree_Old_2	(LPCTSTR path)
{
	v_tstring	foundFiles = ::EnumFiles(path) ;
	v_tstring	foundFolds = ::EnumFoldersTree(path) ;
	#ifdef		MessageBar
	MessageBar	bar(_T("Enumerate files ... Old ..."),foundFolds.size()) ;
	#endif
				foundFolds = ::Sort		(foundFolds) ;
				foundFolds = ::Unique	(foundFolds) ;
	for (size_t index=0 ; index<foundFolds.size() ; index++) {
		tstring		subFold = foundFolds[index] ;
		v_tstring	chFiles = ::EnumFiles(subFold.c_str()) ;
		foundFiles.insert(foundFiles.end(),chFiles.begin(),chFiles.end()) ;
	#ifdef		MessageBar
		bar.SetBarMax(foundFolds.size()) ;
		bar.SetBarInc() ;
	#endif
		}
	return	foundFiles ;
	}
*/

////
//	*******************************************************************************
//	Enumerate Files		tree		specified wild card
//	Create	:	2014/12/22
//	Modify	:	2023/07/11
//	*******************************************************************************
inline	v_tstring	EnumFilesTree	(LPCTSTR path,c_tstring& wildName)
{
	v_tstring	foundFiles = ::EnumFilesTree(path) ;
	v_tstring	extraFiles = ::ExtractFiles (foundFiles,wildName) ;
	return	extraFiles ;
	}

inline	v_tstring	EnumFilesTree	(LPCTSTR path,LPCTSTR wildName)		{	return	::EnumFilesTree(path,	tstring(wildName)) ;	}

////
//	*******************************************************************************
//	Enumerate Files		tree		specified wild card		n types
//	Create	:	2020/07/10
//	*******************************************************************************
inline	v_tstring	EnumFilesTree	(LPCTSTR path,c_v_tstring& wilds)
{
	v_tstring	foundFiles = ::EnumFilesTree	(path) ;
	v_tstring	extraFiles ;
	for (size_t index=0 ; index<wilds.size() ; index++) {
		tstring	wildName = wilds[index] ;
		v_tstring	extra= ::ExtractFiles	(foundFiles,wildName) ;
		extraFiles.insert(extraFiles.end(),extra.begin(),extra.end()) ;
		}
	return	extraFiles ;
	}

////
//	*******************************************************************************
//	Enumerate Files		tree		n folders or files
//	Create	:	2023/07/11
//	*******************************************************************************
inline	v_tstring	EnumFilesTree(c_v_tstring& ff_s,LPCTSTR wild_name=_T("*"))
{
	v_tstring	found_files ;
	{
		for (size_t index=0 ; index<ff_s.size() ; index++) {
			tstring	ff_1 = ff_s[index] ;
			if (::File_IsDirectory(ff_1)) {
				v_tstring	enum_files = ::EnumFilesTree(ff_1.c_str(),wild_name) ;
				found_files.insert(found_files.end(),enum_files.begin(),enum_files.end()) ;
				}
			else {
				found_files.push_back(ff_1) ;
				}
			}
		}
//	found_files = ::Sort	(found_files) ;
	found_files = ::Unique	(found_files) ;
	return	found_files ;
	}

