//	*******************************************************************************
//	Name	:	messcon.hxx
//			:	Message bar Console
//	Author	:	I. Nakagawa
//	Create	:	2014/12/24
//	Modify	:	2015/03/20
//	Modify	:	2015/11/19	cerr --> terr
//	Modify	:	2020/11/11	add omp	critical
//	Modify	:	2023/04/26	change	critical name
//	*******************************************************************************

#pragma		once

#ifdef		_MSC_VER
	#pragma	warning	(disable	:	4786	)
#endif
#include	<iostream>

#include	"messbase.hxx"

#define		I_SUPPORT_MESSAGE_CON

class	MessageCon	:	public	MessageBase		{
public:
			MessageCon					(LPCTSTR msg=_T(""),	const size_t max=0) ;
	virtual	~MessageCon					() ;
	virtual	bool		Reset			(void) ;
public:
	virtual	bool		Draw			(void) ;
protected:
	size_t		MaxLength ;
	} ;

#define	MF_B_MAX	100
#define	MD_B_MAX	25
#define	MC_B_MAX	20

////
//	*******************************************************************************
//	Constructor
//	Create	:	2014/12/24
//	Modify	:	2015/03/20		u_32	->	size_t
//	*******************************************************************************
inline	MessageCon::MessageCon	(
		LPCTSTR			msg		//	=	_T("")
	,	const size_t	max		//	=	0
	)
{
//	B_Postion	= 0 ;
	B_Counter	= 0 ;
	M_Counter	= 0 ;
	WasDrawn	= false ;
	MaxLength	= 0 ;
	SetMessage	(msg) ;
	SetBarMax	(max) ;
	}

////
//	*******************************************************************************
//	Destructor
//	Create	:	2014/12/24
//	*******************************************************************************
inline	MessageCon::~MessageCon	()
{
//	if (!B_Message.empty())	{
		Reset() ;
//		}
	}

////
//	*******************************************************************************
//	Draw
//	Create	:	2014/12/24
//	Modify	:	2015/11/19	cerr --> terr
//	Modify	:	2020/11/11	add omp	critical
//	Modify	:	2023/04/26	change	critical name
//	*******************************************************************************
inline	bool	MessageCon::Reset	(void)
{
	if (WasDrawn) {
		tstring	clrStr ;
		clrStr.resize(MaxLength,_T(' ')) ;
		#ifdef	_OPENMP
			#pragma	omp	critical	(MessageCon_Reset)
		#endif
		{
			std::terr	<< 	clrStr.c_str()	<<	_T("\r") ;
			}
		SetLastTick(0) ;
		}
	B_Counter	= 0 ;
	M_Counter	= 0 ;
	WasDrawn	= false ;
	MaxLength	= 0 ;
	return	true ;
	}

////
//	*******************************************************************************
//	Draw
//	Create	:	2014/12/24
//	Modify	:	2015/11/19	cerr --> terr
//	Modify	:	2020/11/11	add omp	critical
//	Modify	:	2023/04/26	change	critical name
//	*******************************************************************************
inline	bool	MessageCon::Draw	(void)
{
//	tstring	cntStr = GetCountStr() ;
	tstring	cntStr = GetCountR_Str() ;
	/*
	{
		cntStr = ::To_tstring(GetBarCount()) + _T(" / ") + ::To_tstring(GetBarMax()) ;
		cntStr = ::QuotM_Add(cntStr,_T(' ')) ;
		cntStr = ::QuotM_Add(cntStr,_T('(')) ;
		}
	*/
	tstring	barStr ;
	{
		barStr.resize(MC_B_MAX,_T('.')) ;
		if (GetBarMax() > 0) {
			/*
			for (size_t index=0 ; index<MC_B_MAX*GetBarCount()/GetBarMax() ; index++) {
				barStr[index] = _T('*') ;
				}
			*/
			for (size_t index=0 ; index<GetCountR_Pos(MC_B_MAX) ; index++) {
				barStr[index] = _T('*') ;
				}
			}
		}
	{
	//	tstring	clrStr ;
	//	clrStr.resize(MaxLength,_T(' ')) ;
	//	std::terr	<< 	clrStr	+	_T("\r") ;
		}
	#ifdef	_OPENMP
		#pragma	omp	critical	(MessageCon_Draw)
	#endif
	{
		tstring	drawStr ;
		drawStr += B_Message	+	_T("    ") ;
		drawStr += cntStr		+	_T("    ") ;
		drawStr += barStr		+	_T("\r") ;
		std::terr	<<	drawStr.c_str() ;
		if (MaxLength < drawStr.size()) {
			MaxLength = drawStr.size() ;
			}
		}
	WasDrawn	= true ;
	return	true ;
	}

