//	*******************************************************************************
//	Name	:	filestat.hxx
//			:	File Status Functions
//	Author	:	I. Nakagawa
//	Create	:	2013/08/09
//	Modify	:	2014/10/08		
//	Modify	:	2014/12/19		Add File_IsDirectory ...
//	Modify	:	2016/07/01		VC 14	Win XP	_stat
//	Modify	:	2016/11/15		Add c_tstring
//	*******************************************************************************

#pragma		once

#ifdef		_MSC_VER
#pragma		warning	(disable	:	4786	)
#endif
#include	<memory>
#include	<cerrno>
#include	<sys/types.h>
#include	<sys/stat.h>

#include	"i_define.hxx"
#include	"tstring.hxx"
#include	"i_debug.hxx"

#ifdef		_MSC_VER
#define		s_stat		struct	_stati64
#define		TSTAT		_tstati64
#else
#define		s_stat		struct	stat
#define		TSTAT		stat
#endif

#ifndef		mode_t
#define		mode_t		unsigned short
#endif

#ifdef		_MSC_VER
#include	"strpath.hxx"
#endif
#ifdef		_MFC_VER
#include	"FileAttr.hxx"
#endif

////
//	*******************************************************************************
//	File Status
//	Create	:	2013/08/28
//	Modify	:	2016/07/01
//	Modify	:	2024/03/04	add ::Path_Normalize
//	*******************************************************************************
class	F_Stat	{
public:
					F_Stat		(LPCTSTR file_name_)	{
									tstring	file_name =  file_name_ ;
									#ifdef	_MSC_VER
									if (::String__Find(file_name,Path_SP_GNU) != tstring::npos) {
										file_name = ::Path_Normalize(file_name) ;
										}
									#endif
									memset(&Status,0,sizeof(s_stat)) ;
									Error = 0 ;
									if (TSTAT(file_name.c_str(),&Status) < 0)	{
									//	::i_Dump(errno) ;
										Error = errno ;
									//	return	;
										}
									if (Error == EINVAL)
									{
										#ifdef		_MFC_VER
										{
											CFileStatus	fs ;	memset(&fs,0,sizeof(CFileStatus)) ;
											if (CFile::GetStatus(file_name.c_str(),fs)) {
												Error = 0 ;
												Status.st_size	= fs.m_size ;
												Status.st_atime = fs.m_atime.GetTime() ;
												Status.st_mtime	= fs.m_mtime.GetTime() ;
												Status.st_ctime	= fs.m_ctime.GetTime() ;
											//	Status.st_mode	= fs.m_attribute ;
												{
													F_Attribute	fa(file_name) ;
												//	if (fa.Is___Flag(FILE_ATTRIBUTE_READONLY))	{	Status.st_mode |= S_	 ;	}
												//	if (fa.Is___Flag(FILE_ATTRIBUTE_HIDDEN))	{	Status.st_mode |= S_	 ;	}
												//	if (fa.Is___Flag(FILE_ATTRIBUTE_SYSTEM))	{	Status.st_mode |= S_	 ;	}
													if (fa.Is___Flag(FILE_ATTRIBUTE_DIRECTORY))	{	Status.st_mode |= S_IFDIR ;	}
													else										{	Status.st_mode |= S_IFREG ;	}
												//	if (fs.m_attribute & ???)					{	Status.st_mode |= S_IFREG ;	}
												//	if (fs.m_attribute & ???)					{	Status.st_mode |= S_IFDIR ;	}
													}
												}
											}
										#endif
										}
									}
public:
	virtual	bool	IsError		(void)	const	{	return	(Error!=0) ;		}
	virtual	u_64	GetSize		(void)	const	{	return	Status.st_size ;	}
	virtual	time_t	GetATime	(void)	const	{	return	Status.st_atime ;	}
	virtual	time_t	GetMTime	(void)	const	{	return	Status.st_mtime ;	}
	virtual	time_t	GetCTime	(void)	const	{	return	Status.st_ctime ;	}
	virtual	mode_t	GetMode		(void)	const	{	return	Status.st_mode ;	}
	virtual	bool	IsRegular	(void)	const	{	return	((Status.st_mode&S_IFMT)==S_IFREG) ;	}
	virtual	bool	IsDirectory	(void)	const	{	return	((Status.st_mode&S_IFMT)==S_IFDIR) ;	}
public:
	static	u_64	GetSize		(LPCTSTR fileName)	{	F_Stat	fs(fileName) ;	return	fs.GetSize () ;		}
	static	time_t	GetATime	(LPCTSTR fileName)	{	F_Stat	fs(fileName) ;	return	fs.GetATime() ;		}
	static	time_t	GetMTime	(LPCTSTR fileName)	{	F_Stat	fs(fileName) ;	return	fs.GetMTime() ;		}
	static	time_t	GetCTime	(LPCTSTR fileName)	{	F_Stat	fs(fileName) ;	return	fs.GetCTime() ;		}
	static	bool	IsRegular	(LPCTSTR fileName)	{	F_Stat	fs(fileName) ;	return	fs.IsRegular() ;	}
	static	bool	IsDirectory	(LPCTSTR fileName)	{	F_Stat	fs(fileName) ;	return	fs.IsDirectory() ;	}
protected:
	s_stat	Status ;
	int		Error ;
	} ;

////
//	*******************************************************************************
//	File Status Functions
//	Create	:	2014/12/19
//	Modify	:	2016/11/15	Add c_tstring
//	*******************************************************************************
inline	u_64	File_GetSize	(LPCTSTR    fileName)	{	return	F_Stat::GetSize		(fileName) ;		}
inline	time_t	File_GetATime	(LPCTSTR    fileName)	{	return	F_Stat::GetATime	(fileName) ;		}
inline	time_t	File_GetMTime	(LPCTSTR    fileName)	{	return	F_Stat::GetMTime	(fileName) ;		}
inline	time_t	File_GetCTime	(LPCTSTR    fileName)	{	return	F_Stat::GetCTime	(fileName) ;		}
inline	bool	File_IsDirectory(LPCTSTR    fileName)	{	return	F_Stat::IsDirectory	(fileName) ;		}
inline	bool	File_IsRegular	(LPCTSTR    fileName)	{	return	F_Stat::IsRegular	(fileName) ;		}

//#ifdef T_STRING
inline	u_64	File_GetSize	(c_tstring& fileName)	{	return	File_GetSize	(fileName.c_str()) ;	}
inline	time_t	File_GetATime	(c_tstring& fileName)	{	return	File_GetATime	(fileName.c_str()) ;	}
inline	time_t	File_GetMTime	(c_tstring& fileName)	{	return	File_GetMTime	(fileName.c_str()) ;	}
inline	time_t	File_GetCTime	(c_tstring& fileName)	{	return	File_GetCTime	(fileName.c_str()) ;	}
inline	bool	File_IsDirectory(c_tstring& fileName)	{	return	File_IsDirectory(fileName.c_str()) ;	}
inline	bool	File_IsRegular	(c_tstring& fileName)	{	return	File_IsRegular	(fileName.c_str()) ;	}
//#endif
