//	*******************************************************************************
//	Name	:	i_debug.hxx
//			:	Debug Functions
//	Author	:	I. Nakagawa
//	Create	:	2013/03/07
//	Modify	:	2014/01/16
//	Modify	:	2014/12/22
//	Modify	:	2016/10/19
//	Modify	:	2022/08/02	_MSC_VER and _MSC_FULL_VER
//	*******************************************************************************

#pragma		once

#ifdef		_DEBUG
#ifdef		_MSC_VER
#pragma		warning	(disable	:	4786	)
#endif
#include	<iostream>
#endif

#include	"straryfn.hxx"
#include	"xtot.hxx"

////
//	*******************************************************************************
//	Get Compile Macro
//	Create	:	2013/03/07
//	Modify	:	2014/01/16	_WIN64 , _AFXDLL
//	Modify	:	2014/12/25	_WINDOWS , _CONSOLE
//	Modify	:	2016/05/09	_MSC_FULL_VER
//	Modify	:	2016/10/19	_DLL
//	Modify	:	2022/08/02	_MSC_VER and _MSC_FULL_VER
//	*******************************************************************************
inline	std::vector<tstring>	Debug_GetMacroCC_v	(void)
{
	std::vector<tstring>	typeAry ;
	#ifdef	_WIN64
		typeAry.push_back(_T("_WIN64 ")) ;
	#elif	_WIN32
		typeAry.push_back(_T("_WIN32 ")) ;
	#endif
	#ifdef	__GNUC__
		typeAry.push_back(_T("__GNUC__ ")) ;
	#endif
	#ifdef	__APPLE_CC__
		typeAry.push_back(_T("__APPLE_CC__ ")) ;
	#endif
	#ifdef	MONODEVELOP
		typeAry.push_back(_T("MONODEVELOP ")) ;
	#endif
	#ifdef	__MACOS_CLASSIC__
		typeAry.push_back(_T("__MACOS_CLASSIC__ ")) ;
	#endif
	#ifdef	TARGET_OS_MAC
		typeAry.push_back(_T("TARGET_OS_MAC ")) ;
	#endif
	#ifdef	TARGET_OS_WIN32
		typeAry.push_back(_T("TARGET_OS_WIN32 ")) ;
	#endif
	#ifdef	_MSC_VER
		typeAry.push_back(_T("_MSC_VER ")		+	::utot(_MSC_VER,		10)+_T(" ")) ;
	#endif
	#ifdef	_MSC_FULL_VER
		typeAry.push_back(_T("_MSC_FULL_VER ")	+	::utot(_MSC_FULL_VER,	10)+_T(" ")) ;
	#endif
	#ifdef	_MFC_VER
		typeAry.push_back(_T("_MFC_VER ")		+	::utot(_MFC_VER,		16)+_T(" ")) ;
	#endif
	#ifdef	_ATL_VER
		typeAry.push_back(_T("_ATL_VER ")		+	::utot(_ATL_VER,		16)+_T(" ")) ;
	#endif
	#ifdef	_UNICODE
		typeAry.push_back(_T("_UNICODE ")) ;
	#endif
	#ifdef	_MBCS
		typeAry.push_back(_T("_MBCS ")) ;
	#endif
	#ifdef	__cplusplus_cli
		typeAry.push_back(_T("__cplusplus_cli ")) ;
	#endif
	#ifdef	__VERSION__
		typeAry.push_back(_T("__VERSION__ ")	__VERSION__	_T(" ")) ;
	#endif
	#ifdef	_AFXDLL
		typeAry.push_back(_T("_AFXDLL ")) ;
	#elif	_DLL
		typeAry.push_back(_T("_DLL ")) ;
	#endif
	#ifdef	_DEBUG
		typeAry.push_back(_T("_DEBUG ")) ;
	#endif
	#ifdef	DEBUG
		typeAry.push_back(_T("DEBUG ")) ;
	#endif
	#ifdef	_WINDOWS
		typeAry.push_back(_T("_WINDOWS ")) ;
	#endif
	#ifdef	_CONSOLE
		typeAry.push_back(_T("_CONSOLE ")) ;
	#endif
	#ifdef	_DOS
		typeAry.push_back(_T("_DOS ")) ;
	#endif
	#ifdef	_OPENMP
		typeAry.push_back(_T("_OPENMP ")) ;
	#endif
	return	typeAry ;
	}

inline	tstring	Debug_GetMacroCC	(void)
{
	std::vector<tstring>	typeAry = Debug_GetMacroCC_v() ;
	return	::String_Join(typeAry) ;
	}

////
//	*******************************************************************************
//	Debug Dump
//	Create	:	2013/08/09
//	*******************************************************************************
inline	bool	i_Dump	(const int val)
{
	#ifdef	_DEBUG
		#ifdef	_MFC_VER
			afxDump		<< val << _T("\r\n") ;
		#else
			std::terr	<< val << std::endl ;
		#endif
	#endif
	return	true ;
	}

////
//	*******************************************************************************
//	Debug Dump
//	Create	:	2016/10/19
//	*******************************************************************************
inline	bool	i_Dump	(LPCTSTR str)
{
	#ifdef	_DEBUG
		#ifdef	_MFC_VER
			afxDump		<< str << _T("\r\n") ;
		#else
			std::terr	<< str << std::endl ;
		#endif
	#endif
	return	true ;
	}
inline	bool	i_Dump	(c_tstring& str)	{	return	::i_Dump(str.c_str()) ;		}

////
//	*******************************************************************************
//	pause
//	Create	:	2021/05/12
//	*******************************************************************************
inline	bool	pause	(const long timeout=5)
{
	tstring	cmd = _T("timeout ") + ::To_tstring(timeout) ;
	::_tsystem(cmd.c_str()) ;
	return	true ;
	}

