//	*******************************************************************************
//	Name	:	messbase.hxx
//			:	Message bar Base
//	Author	:	I. Nakagawa
//	Create	:	2014/10/14
//	Modify	:	2014/12/24
//	Modify	:	2015/01/08
//	Modify	:	2015/01/15
//	Modify	:	2015/11/11
//	Modify	:	2016/03/29	warning C4267 size_t -> u_32
//	Modify	:	2016/12/16	QuotM -> Bracket
//	Modify	:	2024/07/01	Add include i_vector.hxx
//	*******************************************************************************

#pragma		once

#include	<cmath>

#include	"i_vector.hxx"
#include	"t_tstrng.hxx"
#include	"gettickc.hxx"
#include	"quotm.hxx"
#include	"bracket.hxx"

class	MessageBase		{
public:
						MessageBase		(LPCTSTR msg=_T(""),const size_t max=0) ;
	virtual				~MessageBase	() ;
	virtual	bool		Reset			(void) ;
	virtual	bool		SetBar			(LPCTSTR msg,		const size_t max)	{	SetMessage(msg) ;
																					SetBarMax(max) ;		return	true ;	}
	virtual	bool		SetBarMax		(					const size_t max);//{	M_Counter = max ;		return	true ;	}
	virtual	bool		SetMessage		(LPCTSTR msg)							{	B_Message = msg ;		return	true ;	}
//	virtual	bool		SetWarning		(LPCTSTR msg,const long timer=100) ;
//	virtual	bool		SetError		(LPCTSTR msg,const long timer=500) ;
public:
	virtual	bool		SetBarInc		(void) ;
	virtual	bool		SetBarCount		(const size_t cnt) ;
public:
//	virtual	double		GetBarPosition	(void)	const	{	return	double(GetBarCount())/GetBarMax() ;	}
	virtual	size_t		GetBarCount		(void)	const	{	return	B_Counter ;		}
	virtual	size_t		GetBarMax		(void)	const	{	return	M_Counter ;		}
protected:
	virtual	double		GetCountPos		(void)	const ;
	virtual	double		GetCountR_Pos	(void)	const ;
	virtual	tstring		GetCountStr		(void)	const ;
	virtual	tstring		GetCountR_Str	(void)	const ;
	virtual	size_t		GetCountPos		(const size_t m)	const	{	return	size_t(GetCountPos	()*m) ;		}
	virtual	size_t		GetCountR_Pos	(const size_t m)	const	{	return	size_t(GetCountR_Pos()*m) ;		}
protected:
	virtual	bool		Draw			(void)			{	return	false ;			}
protected:
	static	bool		CheckDraw		(void) ;
protected:
	tstring			B_Message ;		//	message
//	size_t			B_Postion ;		//	0 - 99
	size_t			B_Counter ;		//	0 - M_Counter-1
	size_t			M_Counter ;		//	Max counter
protected:
	bool			WasDrawn ;
protected:
//	static	long		GetLastTick		(void)				{	return	Last_Tick ;		}
//	static	bool		SetLastTick		(const long   t)	{	Last_Tick = t ;		return	true ;	}
	static	double		GetLastTick		(void)				{	return	Last_Tick ;		}
	static	bool		SetLastTick		(const double   t)	{	Last_Tick = t ;		return	true ;	}
protected:
	static	long		GetLevel		(void)				{	return	NestLevel ;		}
public:
	static	bool		ResetLevel		(void)				{	return	SetLevel(0) ;	}
protected:
	static	bool		SetLevel		(const long l)		{	NestLevel = l ;		return	true ;	}
	static	bool		IncLevel		(void)				{	NestLevel++ ;		return	true ;	}
	static	bool		DecLevel		(void)				{	NestLevel-- ;		return	true ;	}
protected:
	static	tstring		GetMessageR		(void)				{	return	R_Message ;		}
	static	size_t		GetCounterR		(void)				{	return	R_Counter ;		}
	static	size_t		GetCountMaxR	(void)				{	return	RMCounter ;		}
protected:
	static	bool		SetMessageR		(c_tstring m)		{	R_Message = m ;		return	true ;	}
	static	bool		SetCounterR		(const size_t c)	{	R_Counter = c ;		return	true ;	}
	static	bool		IncCounterR		(void)				{	R_Counter++ ;		return	true ;	}
	static	bool		SetCountMaxR	(const size_t m)	{	RMCounter = m ;		return	true ;	}
protected:
//	static	long	Last_Tick ;
	static	double	Last_Tick ;
	static	long	NestLevel ;
	static	tstring	R_Message ;		//	message
	static	size_t	R_Counter ;		//	0 - M_Counter-1
	static	size_t	RMCounter ;		//	Max counter
	} ;

////
//	*******************************************************************************
//	Constructor
//	Create	:	2014/12/24
//	Modify	:	2015/01/15
//	*******************************************************************************
inline	MessageBase::MessageBase	(
		LPCTSTR			msg		//	=	_T("")
	,	const size_t	max		//	=	0
	)
{
	if (GetLevel() < 0)	{	ResetLevel() ;	}
	{
		if (GetLevel() == 0)	{
			SetMessageR	(_T("")) ;
			SetCounterR	(0) ;
			SetCountMaxR(0) ;
			}
		IncLevel() ;
		{
			SetCountMaxR(GetCountMaxR()+max) ;
			}
		}
	B_Counter	= 0 ;
	M_Counter	= 0 ;
	WasDrawn	= false ;
	SetMessage	(msg) ;
	SetBarMax	(max) ;
//	if (!GetMessageR().empty() && CheckDraw())	{
//		Draw() ;
//		}
	{
		if (GetLastTick() == 0) {
		//	long	now_Tick	= long(GetTickCount_s()*1000) ;
			double	now_Tick	= GetTickCount_s()*1000. ;
			SetLastTick(now_Tick) ;
			}
		}
	}

////
//	*******************************************************************************
//	Destructor
//	Create	:	2014/12/24
//	*******************************************************************************
inline	MessageBase::~MessageBase			()
{
//	if (!B_Message.empty())	{
		Reset() ;
//		}
	{
		DecLevel() ;
		if (GetLevel() < 0)	{	ResetLevel() ;	}
		if (GetLevel() == 0)	{
			SetMessageR	(_T("")) ;
			SetCounterR	(0) ;
			SetCountMaxR(0) ;
			}
		}
	}

////
//	*******************************************************************************
//	Reset
//	Create	:	2014/12/24
//	*******************************************************************************
inline	bool	MessageBase::Reset			(void)
{
	B_Counter	= 0 ;
	M_Counter	= 0 ;
	WasDrawn	= false ;
	return	true ;
	}

////
//	*******************************************************************************
//	Set Counter Max
//	Create	:	2014/12/24
//	Modify	:	2015/11/11
//	*******************************************************************************
inline	bool	MessageBase::SetBarMax	(const size_t max)
{
	if (max < 10)	{	return	false ;		}	//	2015/11/11	Add
	{
		SetCountMaxR(GetCountMaxR()+(max-GetBarMax())) ;
		}
	M_Counter = max ;
	return	true ;
	}

////
//	*******************************************************************************
//	Increment Counter
//	Create	:	2014/12/24
//	Modify	:	2014/12/26
//	*******************************************************************************
inline	bool	MessageBase::SetBarInc		(void)
{
	if (GetBarMax() == 0)					{	return	false ;	}
	if (GetBarCount() < GetBarMax())		{	B_Counter++ ;	}
	else									{	B_Counter = 0 ;	}		//	reset
	IncCounterR() ;
	{
		long	lastPos = long(100*(GetBarCount()-1)/GetBarMax()) ;
		long	new_Pos = long(100*(GetBarCount()-0)/GetBarMax()) ;
		if (lastPos == new_Pos)				{	return	true ;	}
		}
	#ifdef	_WINDOWS
	#ifdef	_MFC_VER
	#ifdef	_OPENMP
		if (AfxGetMainWnd() == NULL)		{	return	false ;	}
	#endif
	#endif
	#endif
	return	SetBarCount(B_Counter) ;
	}

////
//	*******************************************************************************
//	Update Counter
//	Create	:	2014/12/24
//	Modify	:	2015/01/08
//	*******************************************************************************
inline	bool	MessageBase::SetBarCount	(const size_t cnt)
{
	if (cnt > GetBarMax())					{	return	false ;	}
	B_Counter = cnt ;
	if (GetCounterR() < GetBarCount()) {
		SetCounterR(GetBarCount()) ;
		}
	if (!CheckDraw())						{	return	true ;	}
	return	Draw() ;
	}

////
//	*******************************************************************************
//	Check Draw timing
//	Create	:	2014/12/24
//	Modify	:	2015/01/15
//	*******************************************************************************
inline	bool	MessageBase::CheckDraw		(void)
{
	#define	MB_DrawTiming	100				//	m sec
	long	drawTiming	= MB_DrawTiming ;
//	long	now_Tick	= long(GetTickCount_s()*1000) ;
	double	now_Tick	= GetTickCount_s()*1000. ;
//	if ((::abs(now_Tick-GetLastTick())) < drawTiming) {
//		return	false ;
//		}
	if ((::fabs(now_Tick-GetLastTick())) < drawTiming) {
		return	false ;
		}
	SetLastTick(now_Tick) ;
	return	true ;
	}

////
//	*******************************************************************************
//	Counter position
//	Create	:	2014/12/25
//	*******************************************************************************
inline	double	MessageBase::GetCountPos	(void)		const
{
	double	cntPos = 0. ;
	if (GetBarMax() > 0 &&	GetBarCount() <= GetBarMax()) {
			cntPos = 1. *	GetBarCount() /  GetBarMax() ;
			}
	return	cntPos ;
	}

////
//	*******************************************************************************
//	Counter position	root
//	Create	:	2014/12/25
//	*******************************************************************************
inline	double	MessageBase::GetCountR_Pos	(void)		const
{
	double	cntPos = 0. ;
	if (GetCountMaxR()>0 &&	GetCounterR() <= GetCountMaxR()) {
			cntPos = 1. *	GetCounterR() /  GetCountMaxR() ;
			}
	return	cntPos ;
	}

////
//	*******************************************************************************
//	Counter string
//	Create	:	2014/12/24
//	*******************************************************************************
inline	tstring	MessageBase::GetCountStr	(void)		const
{
	tstring	cntStr ;
	cntStr = ::To_tstring(u_32(GetBarCount())) + _T(" / ") + ::To_tstring(u_32(GetBarMax())) ;
	cntStr = ::QuotM_Add(cntStr,_T(' ')) ;
	cntStr = Bracket_Add(cntStr,_T('(')) ;
	return	cntStr ;
	}

////
//	*******************************************************************************
//	Counter string		root
//	Create	:	2014/12/25
//	Modify	:	2015/01/08
//	*******************************************************************************
inline	tstring	MessageBase::GetCountR_Str	(void)		const
{
	tstring	cntStr ;
	if (GetCountMaxR() > 1) {
		cntStr = ::To_tstring(u_32(GetCounterR())) + _T(" / ") + ::To_tstring(u_32(GetCountMaxR())) ;
		cntStr = ::QuotM_Add(cntStr,_T(' ')) ;
		cntStr = Bracket_Add(cntStr,_T('(')) ;
		}
	return	cntStr ;
	}

