//	*******************************************************************************
//	Name	:	stringfn.hxx
//			:	String Functions
//	Author	:	I. Nakagawa
//	Create	:	2013/03/06
//	Modify	:	2013/09/24
//			:	2013/12/24
//			:	2014/05/09
//			:	2014/10/10	Add Char Code functions
//			:	2015/07/07	Add String_TrimBoth
//			:	2016/05/25	Add String_Encode / Decode
//			:	2017/03/02	Add String_Change
//			:	2019/10/17	bug String_Change	if src_ == dst_
//			:	2020/02/07	Add String_TrimSpace
//			:	2020/02/28	Add Numeric_delete_comma
//			:	2021/06/08	Add String_Change
//			:	2022/08/01	Add String_version
//			:	2024/07/01	Add String_IsNumeric_F
//	*******************************************************************************

#pragma		once

#ifdef		_MSC_VER
#pragma		warning	(disable	:	4786	)
#endif
#include	<algorithm>

#include	<locale>
#include	"i_define.hxx"
#include	"_tdefine.hxx"
#include	"tstring.hxx"
#include	"_s_func.hxx"
#include	"i_vector.hxx"
#include	"v_tstrng.hxx"
#include	"ttox.hxx"
#include	"t_tstrng.hxx"
#include	"tstrmbwc.hxx"

#ifndef		_istdigit
	#define	_istdigit	isdigit
#endif
#ifndef		_istspace
	#define	_istspace	isspace
#endif
#ifndef		_istalpha
	#define	_istalpha	isalpha
#endif
#ifndef		_istcntrl
	#define	_istcntrl	iscntrl
#endif
#ifdef		_MBCS
#include	"MB_Find.hxx"
#endif

////
//	*******************************************************************************
//	Is 7 bit ?
//	Create	:	2013/03/06
//	Modify	:	2013/03/07
//	*******************************************************************************
inline	bool	String_IsOnly7bit	(c_tstring& str_)
{
	tstring	str = str_ ;
	for (size_t index=0 ; index<str.length() ; index++) {
		TCHAR	ch = str[index] ;
		if (ch < 0 || 0x7f < ch)	{
			return	false ;
			}
		}
	return	true ;
	}

////
//	*******************************************************************************
//	Get Last char
//	Create	:	2013/03/01
//	Modify	:	2013/03/04
//	*******************************************************************************
inline	TCHAR	String_GetLast_TC	(c_tstring& str)
{
	if (str.length() == 0)	{	return	0 ;	}
	TCHAR	lastC = str.at(str.length()-1) ;
	#ifdef	_MBCS
	{
		std::vector<TCHAR>	tmpStr ;
		tmpStr.resize(str.length()+1) ;
		::TcsCpy(&tmpStr[0],str.length()+1,str.c_str()) ;
		_tcsrev(&tmpStr[0]) ;
		lastC = tmpStr.at(0) ;
		}
	#endif
	return	lastC ;
	}

////
//	*******************************************************************************
//	Trim Right
//	Create	:	2013/03/12
//	*******************************************************************************
inline	tstring	String_TrimRight	(c_tstring& str_,const TCHAR tc=_T('\0'))
{
	tstring	str = str_ ;
	{
		int		startIndex = -1 ;
		for (size_t index=0 ; index<str.length() ; index++) {
			TCHAR	ch = str[index] ;
			#ifdef	_MBCS
			{
				if (_ismbblead(ch))	{	startIndex=-1 ;	index++ ;	continue ;	}
				}
			#endif
			bool	isTrimChar = false ;
			if		(tc==_T('\0') && _istspace(ch))	{	isTrimChar = true ;			}
			else if (tc == ch)						{	isTrimChar = true ;			}
			else									{	startIndex = -1 ;			}
			if (startIndex < 0 && isTrimChar)		{	startIndex = int(index) ;	}
			}
		if (0 <= startIndex)	{
			str = str.substr(0,startIndex) ;
			}
		}
	/*
	#ifdef	_MFC_VER
	{
		CString	tmp = str_.c_str() ;
		if (tc == _T('\0'))	{	tmp.TrimRight() ;	}
		else				{	tmp.TrimRight(tc) ;	}
		ASSERT(str.c_str()==tmp);
		str = tmp ;
		}
	#endif
	*/
	return	str ;
	}

////
//	*******************************************************************************
//	Trim Left
//	Create	:	2013/03/12
//	*******************************************************************************
inline	tstring	String_TrimLeft	(c_tstring& str_,const TCHAR tc=_T('\0'))
{
	tstring	str = str_ ;
	{
		int		otherIndex = -1 ;
		bool	isOnlySP = true ;
		for (size_t index=0 ; index<str.length() ; index++) {
			TCHAR	ch = str[index] ;
			if		(tc==_T('\0') && _istspace(ch))	{	;	}
			else if (tc == ch)						{	;	}
			else									{
				otherIndex = int(index) ;
				isOnlySP = false ;
				break ;
				}
			}
		if (0 <= otherIndex)	{
			str = str.substr(otherIndex) ;
			}
		else if (isOnlySP) {
			str.erase() ;
			}
		}
	/*
	#ifdef	_MFC_VER
	{
		CString	tmp = str_.c_str() ;
		if (tc == _T('\0'))	{	tmp.TrimLeft() ;	}
		else				{	tmp.TrimLeft(tc) ;	}
		ASSERT(str.c_str()==tmp);
		str = tmp ;
		}
	#endif
	*/
	return	str ;
	}

////
//	*******************************************************************************
//	Trim Left and Right
//	Create	:	2015/07/08
//	*******************************************************************************
inline	tstring	String_TrimBoth	(c_tstring& str_,const TCHAR tc=_T('\0'))
{
	tstring	str = str_ ;
			str = ::String_TrimLeft	(str) ;
			str = ::String_TrimRight(str) ;
	return	str ;
	}

////
//	*******************************************************************************
//	Trim Space
//	Create	:	2020/02/07
//	*******************************************************************************
inline	tstring	String_TrimSpace	(c_tstring& str_)
{
	tstring		str ;
	{
		tstring	tmp = ::String_TrimBoth(str_) ;
		for (size_t index=0 ; index<tmp.length() ; index++) {
			tstring	c ;
			if		(_istlead (tmp[index]))	{	c = tmp.substr(index,2) ;	index++ ;	}
			else if	(_istspace(tmp[index]))	{	;										}
			else							{	c = tmp[index] ;						}
			str += c ;
			}
		}
	return	str ;
	}

////
//	*******************************************************************************
//	Find
//	Create	:	2013/08/30
//	Modify	:	2013/12/24
//	*******************************************************************************
inline	tstring::size_type	String__Find	(c_tstring& tstr,TCHAR ch,tstring::size_type pos=0)
{
	#ifdef	_MBCS
		return	MB_String_Find(tstr,ch,pos) ;
	#else
		return	tstr.find(ch,pos) ;
	#endif
	}

////
//	*******************************************************************************
//	Reverse Find
//	Create	:	2013/08/30
//	Modify	:	2013/12/24
//	*******************************************************************************
inline	tstring::size_type	String_RFind	(c_tstring& tstr,TCHAR ch)
{
	#ifdef	_MBCS
		return	MB_StringRFind(tstr,ch) ;
	#else
		return	tstr.rfind(ch) ;
	#endif
	}

////
//	*******************************************************************************
//	Find first of
//	Create	:	2013/12/24
//	*******************************************************************************
inline	tstring::size_type	String_FindFirstOf	(c_tstring& tstr,LPCTSTR cs,tstring::size_type pos=0)
{
	#ifdef	_MBCS
		return	MB_String_FindFirstOf(tstr,cs,pos) ;
	#else
		return	tstr.find_first_of(cs,pos) ;
	#endif
	}

////
//	*******************************************************************************
//	To Lower
//	Create	:	2013/03/12
//	Modify	:	2014/05/09		Bug	TcsLwr length+1
//	*******************************************************************************
inline	tstring	String_ToLower	(c_tstring& str_)
{
	tstring	str = str_ ;
	#ifdef	__APPLE_CC__
		std::transform(str.begin(),str.end(),str.begin(),tolower) ;
	#else
//	#ifdef	_MSC_VER
	{
		std::vector<TCHAR>	tmpStr ;
		tmpStr.resize(str.length()+1) ;
		::TcsCpy(&tmpStr[0],str.length()+1,str.c_str()) ;
		::TcsLwr(&tmpStr[0],str.length()+1) ;
		str = &tmpStr[0] ;
		}
//	#else
	//	#ifndef	MONODEVELOP
	//		std::transform(str.begin(),str.end(),str.begin(),tolower) ;
	//	#endif
	#endif
	return	str ;
	}
////
//	*******************************************************************************
//	To Upper
//	Create	:	2013/03/12
//	Modify	:	2014/05/09		Bug	TcsUpr length+1
//	*******************************************************************************
inline	tstring	String_ToUpper	(c_tstring& str_)
{
	tstring	str = str_ ;
	#ifdef	__APPLE_CC__
		std::transform(str.begin(),str.end(),str.begin(),toupper) ;
	#else
//	#ifdef	_MSC_VER
	{
		std::vector<TCHAR>	tmpStr ;
		tmpStr.resize(str.length()+1) ;
		::TcsCpy(&tmpStr[0],str.length()+1,str.c_str()) ;
		::TcsUpr(&tmpStr[0],str.length()+1) ;
		str = &tmpStr[0] ;
		}
//	#else
	//	#ifndef	MONODEVELOP
	//		std::transform(str.begin(),str.end(),str.begin(),toupper) ;
	//	#endif
	#endif
	return	str ;
	}

////
//	*******************************************************************************
//	Change File Separator 
//	Create	:	2013/09/03
//	Modify	:	2016/12/15	Add # %
//	*******************************************************************************
inline	tstring	String_ChangePathSP	(c_tstring& name)
{
	tstring	newName = name ;
	for (size_t index=0 ; index<newName.length() ; index++) {
		#ifdef	_MBCS
		if (  _istlead(newName[index]))	{	//	Skip 2-byte character
			index++ ;
			continue ;
			}
		#endif
		TCHAR	c = newName[index] ;
		switch	(c) {
			case	'\\' :		c = '_' ;		break ;
			case	'/' :		c = '_' ;		break ;
			case	':' :		c = '_' ;		break ;
			case	',' :		c = '.' ;		break ;
			case	';' :		c = '.' ;		break ;
			case	'*' :		c = 'x' ;		break ;
			case	'?' :		c = '_' ;		break ;
			case	'\"' :		c = '.' ;		break ;
			case	'<' :		c = '(' ;		break ;
			case	'>' :		c = ')' ;		break ;
			case	'|' :		c = '.' ;		break ;
			case	' ' :		c = '_' ;		break ;
			case	'#' :		c = '-' ;		break ;
			case	'%' :		c = '-' ;		break ;
		//	case	'&' :		c = '-' ;		break ;
		//	case	'@' :		c = '-' ;		break ;
			case	'\r' :		c = '.' ;		break ;
			case	'\n' :		c = '.' ;		break ;
			case	'\t' :		c = '.' ;		break ;
			default :			continue ;		break ;
			}
		newName[index] = c ;
		}
	return	newName ;
	}

////
//	*******************************************************************************
//	change name 7 bit
//	Create	:	2017/03/24
//	Modify	:	2017/04/21	'0'-'9' , 'A'-'Z' , 'a'-'z' , '_' , '-' , '.'
//	*******************************************************************************
inline	tstring	String_Change_7bit	(c_tstring& name)
{
	tstring	newName = name ;
	for (size_t index=0 ; index<newName.length() ; index++) {
		TCHAR	c = newName[index] ;
		c = c & 0x7f ;
		if (c < 0x20)	{		c+= 0x20 ;	}
		if (c > 0x7e)	{		c = 0x7e ;	}
		switch	(c) {
			case	' ' :		c = '_' ;		break ;			//	20
			case	'!' :		c = '1' ;		break ;			//	21
			case	'\"' :		c = '2' ;		break ;			//	22
			case	'#' :		c = '3' ;		break ;			//	23
			case	'$' :		c = '4' ;		break ;			//	24
			case	'%' :		c = '5' ;		break ;			//	25
			case	'&' :		c = '6' ;		break ;			//	26
			case	'\'' :		c = '7' ;		break ;			//	27
			case	'(' :		c = '8' ;		break ;			//	28
			case	')' :		c = '9' ;		break ;			//	29
			case	'*' :		c = 'a' ;		break ;			//	2a
			case	'+' :		c = 'b' ;		break ;			//	2b
			case	',' :		c = 'c' ;		break ;			//	2c
			case	'-' :		c = '-' ;		break ;			//	2d
			case	'.' :		c = '.' ;		break ;			//	2e
			case	'/' :		c = 'f' ;		break ;			//	2f
			case	':' :		c = 'a' ;		break ;			//	3a
			case	';' :		c = 'b' ;		break ;			//	3b
			case	'<' :		c = 'c' ;		break ;			//	3c
			case	'=' :		c = 'd' ;		break ;			//	3d
			case	'>' :		c = 'e' ;		break ;			//	3e
			case	'?' :		c = 'f' ;		break ;			//	3f
			case	'@' :		c = '0' ;		break ;			//	40
			case	'[' :		c = 'b' ;		break ;			//	5b
			case	'\\' :		c = 'c' ;		break ;			//	5c
			case	']' :		c = 'd' ;		break ;			//	5d
			case	'^' :		c = 'e' ;		break ;			//	5e
			case	'_' :		c = '_' ;		break ;			//	5f
			case	'`' :		c = '0' ;		break ;			//	60
			case	'{' :		c = 'b' ;		break ;			//	7b
			case	'|' :		c = 'c' ;		break ;			//	7c
			case	'}' :		c = 'd' ;		break ;			//	7d
			case	'~' :		c = 'e' ;		break ;			//	7e
			case	'\r' :		c = '.' ;		break ;
			case	'\n' :		c = '.' ;		break ;
			case	'\t' :		c = '.' ;		break ;
			default :							break ;
			}
		newName[index] = c ;
		}
	return	newName ;
	}

////
//	*******************************************************************************
//	Check Fixed Value
//	Create	:	2013/09/24
//	*******************************************************************************
inline	bool	String_IsValue	(c_tstring& str_)
{
	tstring	str = str_ ;
			str = ::String_TrimLeft	(str) ;
			str = ::String_TrimRight(str) ;
	for (size_t index=0 ; index<str.length() ; index++) {
		TCHAR	c = str[index] ;
		if (index == 0 && c == _T('-'))	{	continue ;			}		//	minus
		if (c == _T('.'))				{	continue ;			}		//	dot
		if ( _istdigit(c))				{	continue ;			}
		if ( _istspace(c))				{	continue ;			}		//	space
		if ( _istalpha(c))				{	return	false ;		}
		if (!_istdigit(c))				{	return	false ;		}
		}
	return	true ;
	}

////
//	*******************************************************************************
//	Check Numeric
//	Create	:	2014/05/30
//	Modify	:	2017/01/30	Add ','
//	*******************************************************************************
inline	bool	String_IsNumeric	(c_tstring& str_)
{
	tstring	str = str_ ;
			str = ::String_TrimLeft	(str) ;
			str = ::String_TrimRight(str) ;
	for (size_t index=0 ; index<str.length() ; index++) {
		TCHAR	c = str[index] ;
		if (0<c && c<0xff) {
			if (c == ' ')		{	continue ;		}
			if (c == '.')		{	continue ;		}	//	dot
			if (c == ',')		{	continue ;		}	//	comma
			if (c == '-')		{	continue ;		}	//	minus
			if (c == '"')		{	continue ;		}	//	"1,234"
			if (c == ',')		{	continue ;		}
			if (c == '\'')		{	continue ;		}	//	'1,234'
			if (c == 'e')		{	continue ;		}	//	1.234e+10
			if (c == 'E')		{	continue ;		}
			if (c == '+')		{	continue ;		}
			if (_istdigit(c))	{	continue ;		}
			}
		return	false ;
		break ;
		}
	return	true ;
	}

////
//	*******************************************************************************
//	Check Numeric	formula
//	Create	:	2024/06/27
//	*******************************************************************************
inline	bool	String_IsNumeric_F	(c_tstring& str_)
{
	tstring	str = str_ ;
			str = ::String_TrimLeft	(str) ;
			str = ::String_TrimRight(str) ;
	bool	in_E = false ;
	for (size_t index=0 ; index<str.length() ; index++) {
		TCHAR	c = str[index] ;
		if (0<c && c<0xff) {
			if (index == 0 || in_E) {
				if (c == '-')		{	continue ;		}	//	minus
				if (c == '+')		{	continue ;		}
				}
			in_E = false ;
			{
				if (c == ' ')		{	continue ;		}
				if (c == '.')		{	continue ;		}	//	dot
			//	if (c == ',')		{	continue ;		}	//	comma
				if (c == '"')		{	continue ;		}	//	"1,234"
				if (c == ',')		{	continue ;		}
				if (c == '\'')		{	continue ;		}	//	'1,234'
				if (_istdigit(c))	{	continue ;		}
				}
			{
				if (index > 0) {
					if (c == 'e' || c == 'E')	{			//	1.234e+10
						in_E = true ;
						continue ;
						}
					}
				}
			}
		return	false ;
		break ;
		}
	return	true ;
	}

////
//	*******************************************************************************
//	delete ','
//	Create	:	2020/02/28
//	*******************************************************************************
inline	tstring	Numeric_delete_comma	(c_tstring& str_)
{
	tstring	str = str_ ;
	if (str.find(_T(',')) == tstring::npos)	{	return	str ;	}
	if (!::String_IsNumeric(str))			{	return	str ;	}
	tstring	del_c ;
	for (size_t index=0 ; index<str.length() ; index++) {
		TCHAR	c = str[index] ;
		if (c == _T(','))		{	continue ;		}	//	comma
		del_c += c ;
		}
	return	del_c ;
	}

////
//	*******************************************************************************
//	Change "\r\n" -> "  "
//	Create	:	2014/06/12
//	*******************************************************************************
inline	tstring	String_CrLfToSP	(LPCTSTR str,LPCTSTR sp=_T("  "))
{
	tstring	spCR = tstring(sp).substr(0,1) ;
	tstring	spLF = tstring(sp).substr(1) ;
	tstring	tmp = str ;
	tstring	newLine ;
	for (size_t index=0 ; index<tmp.length() ; index++) {
		tstring	c = tmp.substr(index,1) ;
		if		(c == _T("\r"))	{	c = spCR ;	}
		else if	(c == _T("\n"))	{	c = spLF ;	}
		newLine += c ;
		}
	return	newLine ;
	}

inline	tstring	String_CrLfToSP	(c_tstring str,LPCTSTR sp=_T("  "))		{
	return	::String_CrLfToSP(str.c_str(),sp) ;
	}

////
//	*******************************************************************************
//	sanitize
//	Create	:	2017/05/25
//	*******************************************************************************
inline	tstring	String_sanitize	(LPCTSTR str)
{
	tstring	tmp = str ;
	tstring	newLine ;
	for (size_t index=0 ; index<tmp.length() ; index++) {
		tstring	c = tmp.substr(index,1) ;
		if		(c == _T("&"))	{	c = _T("&amp;") ;	}
		else if	(c == _T("<"))	{	c = _T("&lt;") ;	}
		else if	(c == _T(">"))	{	c = _T("&gt;") ;	}
		else if	(c == _T("\""))	{	c = _T("&quot;") ;	}
	//	else if	(c == _T("\'"))	{	c = _T("&#039;") ;	}
		newLine += c ;
		}
	return	newLine ;
	}
inline	tstring	String_sanitize	(c_tstring str)	{	return	::String_sanitize(str.c_str()) ;	}

////
//	*******************************************************************************
//	Compare No Case
//	Create	:	2016/08/26
//	Modify	:	2016/09/05		LPCTSTR		->	c_tstring&
//	*******************************************************************************
inline	bool	String_CompareNoCase	(c_tstring& str1,c_tstring& str2)
{
	tstring	tStr1 = str1 ;
	tstring	tStr2 = str2 ;
	if (tStr1 == tStr2)		{	return	true ;	}
	tStr1 = ::String_ToLower(tStr1) ;
	tStr2 = ::String_ToLower(tStr2) ;
	return	(tStr1 == tStr2) ;
	}
/*
inline	bool	String_CompareNoCase	(LPCTSTR str1,LPCTSTR str2)
{
	tstring	tStr1 = str1 ;
	tstring	tStr2 = str2 ;
	if (tStr1 == tStr2)		{	return	true ;	}
	tStr1 = ::String_ToLower(tStr1) ;
	tStr2 = ::String_ToLower(tStr2) ;
	return	(tStr1 == tStr2) ;
	}
inline	bool	String_CompareNoCase	(c_tstring& str1,c_tstring str2)	{
	return	::String_CompareNoCase(str1.c_str(),str2.c_str()) ;
	}
*/

////
//	*******************************************************************************
//	Find No Case
//	Create	:	2014/10/07
//	Modify	:	2016/09/05		LPCTSTR		->	c_tstring&
//	Modify	:	2017/03/02		Add pos
//	*******************************************************************************
inline	tstring::size_type	String_FindNoCase	(c_tstring& str1,c_tstring& str2,tstring::size_type pos=0)
{
	tstring	fStr1 = str1 ;
	size_t	fIndex = fStr1.find(str2,pos) ;
	if (fIndex != tstring::npos)	{	return	fIndex ;	}	//	found String ?
	tstring	fStr2 = str2 ;
	fStr1 = ::String_ToLower(fStr1) ;
	fStr2 = ::String_ToLower(fStr2) ;
	return	fStr1.find(fStr2,pos) ;								//	Find no case
	}
/*
inline	tstring::size_type	String_FindNoCase	(LPCTSTR str1,LPCTSTR str2)
{
	tstring	fStr1 = str1 ;
	size_t	fIndex = fStr1.find(str2) ;
	if (fIndex != tstring::npos)	{	return	fIndex ;	}	//	found String ?
	tstring	fStr2 = str2 ;
	fStr1 = ::String_ToLower(fStr1) ;
	fStr2 = ::String_ToLower(fStr2) ;
	return	fStr1.find(fStr2) ;									//	Find no case
	}
inline	tstring::size_type	String_FindNoCase	(c_tstring& str1,c_tstring str2)	{
	return	::String_FindNoCase(str1.c_str(),str2.c_str()) ;
	}
*/

////
//	*******************************************************************************
//	change
//	Create	:	2017/03/02
//	Modify	:	2017/04/14	bug	if src.len <  dst.len
//	Modify	:	2019/10/17	bug	if src.len <= dst.len		src_ == dst_
//	*******************************************************************************
inline	tstring	String_Change	(LPCTSTR str_,LPCTSTR src_,LPCTSTR dst_)
{
	tstring	str = str_ ;
	tstring	src = src_ ;
	tstring	dst = dst_ ;
	for (size_t index=0 ; index<str.length() ; /*index++*/) {
	//	size_t	found = str.find(src,index) ;
		size_t	found = ::String_FindNoCase(str,src,index) ;
		if (found == tstring::npos)		{	break ;		}
		tstring	pre = str.substr(0,found) ;
		tstring	pst = str.substr(found+src.length()) ;
		str = pre + dst + pst ;
		index = found ;
		if (src.length() <= dst.length()) {
			index = index + (dst.length()-src.length())+1 ;
			}
		}
	return	str ;
	}

////
//	*******************************************************************************
//	change tstring
//	Create	:	2021/06/04
//	*******************************************************************************
inline	tstring	String_Change	(c_tstring& str_,const TCHAR src,const TCHAR dst)
{
	tstring	str = str_ ;
	for (size_t index=0 ; index<str.size() ; index++) {
		TCHAR	ch = str[index] ;
		if (ch == src) {
			str[index] = dst ;
			}
		}
	return	str ;
	}

////
//	*******************************************************************************
//	to 1.23.4567.890
//	Create	:	2022/07/29
//	*******************************************************************************
inline	tstring	String_version	(LPCTSTR ver_s)
{
	tstring	ver_tmp = ver_s ;
	tstring	ver_str ;
	for (size_t index=0 ; index<ver_tmp.size() ; index++) {
		TCHAR	c = ver_tmp[index] ;
		if		(c == _T(' '))	{							continue ;	}	//	skip   ' '
		else if (c == _T(','))	{	ver_str += _T('.') ;	continue ;	}	//	change ','
		else					{	ver_str += c ;			continue ;	}
		}
	return	ver_str ;
	}

////
//	*******************************************************************************
//	Encode		for CF_DSPTEXT
//	Create	:	2016/05/25
//	*******************************************************************************
inline	tstring	String_Encode	(LPCTSTR str,const bool isAll=false)
{
	tstring	codeStr = str ;
	if (codeStr.length() == 0)		{	return	codeStr ;	}
	#ifdef	_UNICODE
		if (codeStr[0] != 0xFEFF) {
			TCHAR	bom = 0xFEFF ;
			codeStr = bom + codeStr ;
			}
	#endif
	for (size_t index=0 ; index<codeStr.length() ; index++) {
		TCHAR	c = codeStr[index] ;
		u_16	w = unsigned(c) ;
		if (isAll) {
			}
		else {
			switch	(w)	{
				case	'\\'	:
			//	case	'?'		:
				case	' '		:
					break ;
				default	:
					if (0x0 <= w && w < 0x80)	{	continue ;		}
					break ;
				}
			}
		#ifdef	_UNICODE
		tstring	newC = _T("\\x") + ::u32to0t(	w,		16,4) ;
		#else
		tstring	newC = _T("\\x") + ::u32to0t(u__8(c),	16,2) ;
		#endif
				newC += _T(" ") ;
		tstring	pre = codeStr.substr(0,index) ;
		tstring	pst = codeStr.substr(index+1) ;
		codeStr = pre + newC + pst ;
		index += newC.length()-1 ;
		}
	return	codeStr ;
	}

////
//	*******************************************************************************
//	Decode
//	Create	:	2016/05/25
//	*******************************************************************************
inline	tstring	String_Decode	(LPCTSTR codeStr)
{
	tstring	str = codeStr ;
	if (str.find(_T("\\x")) == tstring::npos)	{	return	str ;	}	//	no code
	bool	isWC = false ;
	{
		tstring	bomStr = str.substr(0,6) ;
		if (::String_FindNoCase(bomStr.c_str(),_T("\\xfeff")) == 0)	{
			isWC = true ;
			str = str.substr(6+1) ;		//	skip "\xfeff "
			}
		}
	int		lenC = isWC ? 6 : 4 ;	//	Wide : \x005c 	MBCS : \x5c 
			lenC+= 1 ;				//	"\x..." + " "
	std::string		strMB ;			//	BYTE size	MBCS
	std::wstring	strWC ;			//	WORD size	Wide
	for (size_t index=0 ; index<str.length() ; index++) {
		TCHAR	c  = str[index+0] ;
		tstring	c1 = str.substr(index,2) ;		c1 = String_ToLower(c1) ;
		tstring	ps = str.substr(index,lenC) ;
		if (c1==_T("\\x") && ps.length() > 3) {	//	is code ?
			tstring	code = ps.substr(2) ;
			char	newC =    char(::ttou2(code,16)) ;
			wchar_t	newW = wchar_t(::ttou2(code,16)) ;
			strMB += newC ;
			strWC += newW ;
			index+= lenC-1 ;
			}
		else {
			strMB +=    char(c) ;
			strWC += wchar_t(c) ;
			}
		}
	strMB +=    char(0) ;
	strWC += wchar_t(0) ;
	if (isWC)	{	str = ::To_tstring(/*LPCWSTR*/(strWC.c_str())) ;	}
	else		{	str = ::To_tstring(/*LPCSTR */(strMB.c_str())) ;	}
	return	str ;
	}

////
//	*******************************************************************************
//	Change Control code		for make 1 line
//	Create	:	2013/09/03
//	Modify	:	2016/06/01	'\\'	->	"\\x5c"
//	*******************************************************************************
inline	tstring	String_ToCode	(LPCTSTR m_Line)
{
	tstring	mLine = m_Line ;
	tstring	newLine ;
	for (size_t index=0 ; index<mLine.length() ; index++) {
		TCHAR	c = mLine[index] ;
		tstring	newC ;	newC = c ;
		#ifdef	_MBCS
		if (  _istlead(mLine[index]))	{	//	Skip 2-byte character
			index++ ;
			newC += mLine[index] ;
			newLine += newC ;
			continue ;
			}
		#endif
		switch	(c)	{
			case	'\r' :		newC = _T("\\r") ;		break ;
			case	'\n' :		newC = _T("\\n") ;		break ;
			case	'\t' :		newC = _T("\\t") ;		break ;
			case	'\"' :		newC = _T("\\x22");		break ;
			case	'\'' :		newC = _T("\\x27");		break ;
			case	',' :		newC = _T("\\x2c");		break ;
		//	case	'\\' :		newC = _T("\\\\") ;		break ;
			case	'\\' :		newC = _T("\\x5c") ;	break ;
		//	case	' ' :		newC = _T("\\x20");		break ;
			default :									break ;
			}
		newLine += newC ;
		}
	return	newLine ;
	}

////
//	*******************************************************************************
//	Change Control code
//	Create	:	2013/09/03
//	Modify	:	2016/06/01	"\\\\"	->	"\\x5c"
//	*******************************************************************************
inline	tstring	String_ToChar	(LPCTSTR line_1)
{
	tstring	line1 = line_1 ;
	tstring	newLine ;
	for (size_t index=0 ; index<line1.length() ; index++) {
		TCHAR	c = line1[index] ;
		tstring	newC ;	newC = c ;
		#ifdef	_MBCS
		if (  _istlead(line1[index]))	{	//	Skip 2-byte character
			index++ ;
			newC += line1[index] ;
			newLine += newC ;
			continue ;
			}
		#endif
		if (c == '\\')	{
			tstring	ps4 = line1.substr(index,4) ;
			tstring	ps2 = line1.substr(index,2) ;
			if		(ps4 == _T("\\x20"))	{	newC = _T(" ") ;		index+=3 ;	}
			else if	(ps4 == _T("\\x22"))	{	newC = _T("\"") ;		index+=3 ;	}
			else if	(ps4 == _T("\\x27"))	{	newC = _T("\'") ;		index+=3 ;	}
			else if	(ps4 == _T("\\x2c"))	{	newC = _T(",") ;		index+=3 ;	}
			else if	(ps4 == _T("\\x5c"))	{	newC = _T("\\") ;		index+=3 ;	}
			else if	(ps4 == _T("\\x0d"))	{	newC = _T("\r") ;		index+=3 ;	}
			else if	(ps4 == _T("\\x0a"))	{	newC = _T("\n") ;		index+=3 ;	}
			else if	(ps4 == _T("\\x09"))	{	newC = _T("\n") ;		index+=3 ;	}
			else if	(ps2 == _T("\\r"))		{	newC = _T("\r") ;		index+=1 ;	}
			else if	(ps2 == _T("\\n"))		{	newC = _T("\n") ;		index+=1 ;	}
			else if	(ps2 == _T("\\t"))		{	newC = _T("\t") ;		index+=1 ;	}
			else if	(ps2 == _T("\\\\"))		{	newC = _T("\\") ;		index+=1 ;	}
			else							{	newC = _T(" \'Error!\' ") ;			}
			}
		newLine += newC ;
		}
	return	newLine ;
	}

////
//	*******************************************************************************
//	need to code ?
//	Create	:	2016/05/30
//	*******************************************************************************
inline	bool	String_NeedCode	(LPCTSTR str)
{
	bool	foundCtrl = false ;
	tstring	line = str ;
	for (size_t index=0 ; index<line.length() ; index++) {
		#ifdef	_MBCS
		if (  _istlead(line[index]))	{	//	Skip 2-byte character
			index++ ;
			continue ;
			}
		#endif
		TCHAR	c = line[index] ;
		if (::_istcntrl(unsigned(c)))	{	//	check control
			foundCtrl = true ;
			break ;
			}
		}
	return	foundCtrl ;
	}

////
//	*******************************************************************************
//	is coded ?
//	Create	:	2016/05/30
//	Modify	:	2016/06/01	"\\\\"	->	"\\x5c"
//	*******************************************************************************
inline	bool	String_IsCoded	(LPCTSTR str)
{
	if (::String_NeedCode(str))		{	return	false ;	}
	tstring	line = str ;
	bool	isCoded = false ;
	{
		for (size_t index=0 ; index<line.length() ; index++) {
			TCHAR	c = line[index] ;
			if (c == '\\')	{
				tstring	ps4 = line.substr(index,4) ;
				tstring	ps2 = line.substr(index,2) ;
				if		(ps4 == _T("\\x20"))	{	isCoded = true ;	break ;		}	//	' '
				else if	(ps4 == _T("\\x22"))	{	isCoded = true ;	break ;		}	//	'\"'
				else if	(ps4 == _T("\\x27"))	{	isCoded = true ;	break ;		}	//	'\''
				else if	(ps4 == _T("\\x2c"))	{	isCoded = true ;	break ;		}	//	','
				else if	(ps4 == _T("\\x5c"))	{	isCoded = true ;	break ;		}	//	'\\'
				else if	(ps4 == _T("\\x0d"))	{	isCoded = true ;	break ;		}
				else if	(ps4 == _T("\\x0a"))	{	isCoded = true ;	break ;		}
				else if	(ps4 == _T("\\x09"))	{	isCoded = true ;	break ;		}
				else if	(ps2 == _T("\\r"))		{	isCoded = true ;	break ;		}
				else if	(ps2 == _T("\\n"))		{	isCoded = true ;	break ;		}
				else if	(ps2 == _T("\\t"))		{	isCoded = true ;	break ;		}
			//	else if	(ps2 == _T("\\\\"))		{	isCoded = true ;	break ;		}
				}
			}
		}
	return	isCoded ;
	}
////
//	*******************************************************************************
//	change code function	array
//	Create	:	2016/05/30
//	*******************************************************************************
inline	v_tstring	String_ToCode	(c_v_tstring& strAry)
{
	v_tstring	codeAry ;
	for (size_t index=0 ; index<strAry.size() ; index++) {
		tstring	str = strAry[index] ;
		tstring	code = str ;
		if (!::String_IsCoded(str.c_str())) {
				code = ::String_ToCode(str.c_str()) ;
			}
		codeAry.push_back(code) ;
		}
	return	codeAry ;
	}

inline	v_tstring	String_ToChar	(c_v_tstring& strAry)
{
	v_tstring	chrsAry ;
	for (size_t index=0 ; index<strAry.size() ; index++) {
		tstring	str = strAry[index] ;
		tstring	chrs = str ;
		if (::String_IsCoded(str.c_str())) {
				chrs = ::String_ToChar(str.c_str()) ;
			}
		chrsAry.push_back(chrs) ;
		}
	return	chrsAry ;
	}

/*
inline	bool	String_NeedCoded	(c_v_tstring& strAry)
{
	for (size_t index=0 ; index<strAry.size() ; index++) {
		tstring	str = strAry[index] ;
		if (::String_NeedCoded(str))	{	return	true ;	}
		}
	return	false ;
	}

inline	bool	String_IsCoded	(c_v_tstring& strAry)
{
	for (size_t index=0 ; index<strAry.size() ; index++) {
		tstring	str = strAry[index] ;
		if (::String_IsCoded(str))	{	return	true ;	}
		}
	return	false ;
	}
*/

////
#ifdef		____move_text_cc_hxx______________
//	*******************************************************************************
//	define	Char Code
//	Create	:	2012/03/27
//	Modify	:	2020/01/07
//	*******************************************************************************
#define	CC_UTF_8			8
#define	CC_UTF_16			16
#define	CC_UTF_16_BE		17
#define	CC_UTF_32			32
#define	CC_UTF_32_BE		33
#define	CC_Shift_JIS		'J'
#define	CC_ASCII			'A'
#define	CC_Unknown			0
#endif	//	____move_text_cc_hxx______________

#include	"text_cc.hxx"

////
//	*******************************************************************************
//	To Char Code String
//	Create	:	2023/09/21
//	*******************************************************************************
inline	tstring		ToString_char_code	(const long cc)
{
	tstring	cc_str ;
	switch	(cc)	{
		case	CC_ASCII		:	cc_str = _T("ASCII") ;			break ;
		case	CC_UTF_8		:	cc_str = _T("UTF_8") ;			break ;
		case	CC_UTF_16		:	cc_str = _T("UTF_16") ;			break ;
		case	CC_UTF_16_BE	:	cc_str = _T("UTF_16_BE") ;		break ;
		case	CC_UTF_32		:	cc_str = _T("UTF_32") ;			break ;
		case	CC_UTF_32_BE	:	cc_str = _T("UTF_32_BE") ;		break ;
		case	CC_Shift_JIS	:	cc_str = _T("Shift_JIS") ;		break ;
		case	CC_Unknown		:	cc_str = _T("Unknown") ;		break ;
		default					:	cc_str = _T("Other") ;			break ;
		}
	return	cc_str ;
	}

////
//	*******************************************************************************
//	To Char Code String
//	Create	:	2014/10/07
//	*******************************************************************************
inline	tstring		ToString_CharCode	(const long cc)
{
	tstring	ccStr ;
	switch	(cc)	{
		case	CC_UTF_8		:	ccStr = _T("UTF_8") ;			break ;
		case	CC_UTF_16		:	ccStr = _T("UTF_16") ;			break ;
		case	CC_UTF_16_BE	:	ccStr = _T("UTF_16_BE") ;		break ;
		case	CC_Shift_JIS	:	ccStr = _T("Shift_JIS") ;		break ;
		case	CC_Unknown		:	ccStr = _T("Unknown") ;			break ;
		default					:	ccStr = _T("Other") ;			break ;
		}
	return	ccStr ;
	}

////
//	*******************************************************************************
//	Check Char Code		xml
//	Create	:	2014/10/07
//	Modify	:	2015/07/09	VRML V2.0
//	Modify	:	2015/10/29	HTML
//	*******************************************************************************
inline	long	String_GetCharCode	(c_v_tstring strAry)
{
	long		charCode	=	CC_Unknown ;
	if (charCode == CC_Unknown) {
		for (size_t index=0 ; index<strAry.size() && index<10 ; index++) {	//	check xml encoding
			tstring	str1 = strAry[index] ;
			size_t	findXML = str1.find(_T("xml")) ;
			size_t	findEnc = str1.find(_T("encoding"),findXML+1) ;
			if (findXML != tstring::npos && findEnc != tstring::npos) {
						findEnc= str1.find(_T("="),findEnc+1) ;
				tstring	encStr = str1.substr(findEnc+1,20) ;
				if		(::String_FindNoCase(encStr.c_str(),_T("UTF-8")		) != tstring::npos)		{	charCode = CC_UTF_8 ;		}
				else if	(::String_FindNoCase(encStr.c_str(),_T("UTF-16")	) != tstring::npos)		{	charCode = CC_UTF_16 ;		}
				else if	(::String_FindNoCase(encStr.c_str(),_T("Shift_JIS")	) != tstring::npos)		{	charCode = CC_Shift_JIS ;	}
				break ;
				}
			}
		}
	if (charCode == CC_Unknown) {
		for (size_t index=0 ; index<strAry.size() && index<10 ; index++) {	//	check html
			tstring	str1 = strAry[index] ;
			size_t	findHTM = str1.find(_T("html")) ;
			size_t	findEnc = str1.find(_T("charset"),findHTM+1) ;
			if (findHTM != tstring::npos && findEnc != tstring::npos) {
						findEnc= str1.find(_T("="),findEnc+1) ;
				tstring	encStr = str1.substr(findEnc+1,20) ;
				if		(::String_FindNoCase(encStr.c_str(),_T("UTF-8")		) != tstring::npos)		{	charCode = CC_UTF_8 ;		}
				else if	(::String_FindNoCase(encStr.c_str(),_T("UTF-16")	) != tstring::npos)		{	charCode = CC_UTF_16 ;		}
				else if	(::String_FindNoCase(encStr.c_str(),_T("Shift_JIS")	) != tstring::npos)		{	charCode = CC_Shift_JIS ;	}
				break ;
				}
			}
		}
	if (charCode == CC_Unknown) {
		for (size_t index=0 ; index<strAry.size() && index<10 ; index++) {	//	check VRML 2.0
			tstring	str1 = strAry[index] ;
			size_t	findVRML = str1.find(_T("VRML")) ;						//	#VRML V2.0 utf8
			size_t	findX3D_ = str1.find(_T("X3D" )) ;						//	#X3D V3.0 utf8
			if (findVRML != tstring::npos || findX3D_ != tstring::npos) {
				if (str1.find(_T("utf8")) != tstring::npos)											{	charCode = CC_UTF_8 ;		}
				break ;
				}
			}
		}
	return	charCode ;
	}

inline	long	String_GetCharCode	(LPCTSTR txtStr)
{
	v_tstring	strAry ;
	strAry.push_back(txtStr) ;
	return	::String_GetCharCode(strAry) ;
	}

