//	*******************************************************************************
//	Name	:	text_msc.hxx
//			:	load text functions
//	Author	:	I. Nakagawa
//	Create	:	2020/01/07
//	Modify	:	2020/01/30
//	*******************************************************************************

#pragma		once

#include	"text_cc.hxx"
#include	"tstrutf8.hxx"

////
//	*******************************************************************************
//	big to little
//	Create	:	2020/01/07
//	*******************************************************************************
inline	v_char	be_to_le_32	(const v_char& vchr_)
{
	v_char	vchr= vchr_ ;
	v_char	tmp = vchr_ ;
	for (unsigned index=0 ; index<tmp.size()/4 ; index++) {
		char	c1 = tmp[index*4+0] ;
		char	c2 = tmp[index*4+1] ;
		char	c3 = tmp[index*4+2] ;
		char	c4 = tmp[index*4+3] ;
		vchr[index*4+0] = c4 ;
		vchr[index*4+1] = c3 ;
		vchr[index*4+2] = c2 ;
		vchr[index*4+3] = c1 ;
		}
	return	vchr ;
	}

inline	v_char	be_to_le_16	(const v_char& vchr_)
{
	v_char	vchr= vchr_ ;
	v_char	tmp = vchr_ ;
	for (unsigned index=0 ; index<tmp.size()/2 ; index++) {
		char	h = tmp[index*2+0] ;
		char	l = tmp[index*2+1] ;
		vchr[index*2+0] = l ;
		vchr[index*2+1] = h ;
		}
	return	vchr ;
	}

////
//	*******************************************************************************
//	vectro<char>	->	tstring
//	Create	:	2014/02/13
//	Modify	:	2014/03/20
//	*******************************************************************************
inline	v_char	MSC_32_16(const v_char& v_chr)
{
	v_char	vchr ;
	vchr.resize(v_chr.size()) ;
	size_t	newSize = 0 ;
	for (size_t index=0 ; index<v_chr.size() ; index+=2) {
		char	l = v_chr[index+0] ;
		char	h = v_chr[index+1] ;
		if (l == 0 && h == 0)	{	continue ;	}
		vchr[newSize+0] = l ;
		vchr[newSize+1] = h ;
		newSize += 2 ;
		}
	vchr.resize(newSize) ;
	return	vchr ;
	}

////
//	*******************************************************************************
//	to tstring	msc
//	Create	:	2020/01/07
//	Modify	:	2020/01/30	down "\xFF\xFE\x00\x00"		add "check size"
//	*******************************************************************************
inline	tstring	to_tstring_msc	(c_v_char& vchr)
{
	tstring	tstr ;
	{
		v_char	t_chr = vchr ;
		long	cc = ::get_char_code(&t_chr) ;		//	t_chr ... skip BOM
		if (t_chr.size() > 0) {
			switch	(cc)	{
				case	CC_UTF_8		:									break ;
				case	CC_UTF_16		:									break ;
				case	CC_UTF_16_BE	:	t_chr = ::be_to_le_16(t_chr) ;	break ;
				case	CC_UTF_32		:									break ;
				case	CC_UTF_32_BE	:	t_chr = ::be_to_le_32(t_chr) ;	break ;
				case	CC_Shift_JIS	:									break ;
				case	CC_ASCII		:									break ;
				default					:									break ;
				}
			}
		if (t_chr.size() > 0) {
			switch	(cc)	{
				case	CC_UTF_32		:	t_chr = ::MSC_32_16(t_chr) ;	break ;
				case	CC_UTF_32_BE	:	t_chr = ::MSC_32_16(t_chr) ;	break ;
				default					:									break ;
				}
			}
		if (t_chr.size() > 0) {
			LPWSTR	wcP = NULL ;
			LPSTR	u8P = NULL ;
			LPSTR	mbP = NULL ;
			switch	(cc)	{
				case	CC_UTF_16		:
				case	CC_UTF_16_BE	:
				case	CC_UTF_32		:
				case	CC_UTF_32_BE	:	wcP = LPWSTR(&t_chr[0]) ;	tstr = ::To_tstring(wcP) ;		break ;
				case	CC_UTF_8		:	u8P = LPSTR (&t_chr[0]) ;	tstr = ::UTF8_To   (u8P) ;		break ;
				case	CC_Shift_JIS	:	mbP = LPSTR (&t_chr[0]) ;	tstr = ::To_tstring(mbP) ;		break ;
				case	CC_ASCII		:	mbP = LPSTR (&t_chr[0]) ;	tstr = ::To_tstring(mbP) ;		break ;
				default					:	mbP = LPSTR (&t_chr[0]) ;	tstr = ::To_tstring(mbP) ;		break ;
				}
			}
		}
	return	tstr ;
	}

