//	**************************************************************************************************
//	Name	:	xml_out.hxx
//			:	XML out
//	Author	:	I. Nakagawa
//	Create	:	2012/02/01
//	Modify	:	2012/02/16
//			:	2012/09/26	Add "???Comment"
//			:	2014/02/12	MFC	-->	STL
//			:	2015/11/10	Add MessageBar
//			:	2016/04/25	remains of the array
//			:	2019/05/29	change "AddComment"
//			:	2019/08/06	change attribute value quot
//	**************************************************************************************************

#pragma		once

#include	"i_v_type.hxx"
#include	"xml_base.hxx"
#include	"textfile.hxx"
#include	"stringto.hxx"
#include	"messbar.hxx"
#include	"i_trace.hxx"

////
//	*******************************************************************************
//	Xml_Out
//	Create	:	2012/02/16
//	Modify	:	2012/09/26	Add "???Comment"
//			:	2014/02/12	MFC	-->	STL
//	*******************************************************************************
class	Xml_Out		{
public:
	static	bool		Export			(const Xml_E& elements,LPCTSTR xmlName) ;
	static	v_tstring	CreateElement	(const Xml_E& elements,const int indent) ;
public:
	static	bool		AddComment		(Xml_E&	xml_e,c_v_tstring& cmt_ary) ;
	static	bool		AddComment		(Xml_E&	xml_e,LPCTSTR      comment) ;
public:
	static	bool		SetComment		(Xml_E* xml_e,LPCTSTR comment=NULL) ;
	static	bool		SetCommentTree	(Xml_E* xml_e,LPCTSTR comment) ;
protected:
	static	bool		ExportI			(const Xml_E& elements,LPCTSTR xmlName) ;
	} ;

////
//	*******************************************************************************
//	Export
//	Create	:	2012/02/16
//	Modify	:	2012/10/31	Delete MSXML code
//			:	2014/02/12	MFC	-->	STL
//	*******************************************************************************
inline	bool	Xml_Out::Export	(const Xml_E& elements,LPCTSTR xmlName)
{
	{
		return	Xml_Out::	ExportI	(elements,xmlName) ;
		}
	}

////
//	*******************************************************************************
//	Export
//	Create	:	2012/03/22
//	Modify	:	2014/02/12	MFC	-->	STL
//	Modify	:	2014/03/20	Support UTF-8
//	Modify	:	2016/04/22	Use SaveUTF8 v_tstring
//	*******************************************************************************
inline	bool	Xml_Out::ExportI	(const Xml_E& elements,LPCTSTR xmlName)
{
	::i_Trace(_T("Xml_Out::ExportI")) ;
	v_tstring	strAry ;
	{
		tstring	xml_ver = _T("<?xml version=\"1.0\"?>") ;
				xml_ver = _T("<?xml version=\"1.0\" encoding=\"UTF-8\"?>") ;
		#ifdef	_MSC_VER
		//		xml_ver = _T("<?xml version=\"1.0\" encoding=\"Shift_JIS\"?>") ;
		#endif
		strAry.push_back(xml_ver) ;
		}
	::i_Trace(_T("Call Xml_Out::CreateElement")) ;
	{
		v_tstring	sa = CreateElement(elements,0) ;
		strAry.insert(strAry.end(),sa.begin(),sa.end()) ;
		}
	::i_Trace(_T("Call SaveUTF8")) ;
	{
		::SaveUTF8(xmlName,strAry,false) ;
		}
	::i_Trace(_T("End SaveUTF8")) ;
	/*
	tstring	tStr = ::String_Join_Line(strAry) ;
	{
		::SaveUTF8(xmlName,tStr,false) ;
		}
	*/
	return	true ;
	}

////
//	*******************************************************************************
//	CreateElement
//	Create	:	2012/02/16
//	Modify	:	2012/03/21	svg:text , svg:tspan
//			:	2012/10/31	comment
//			:	2014/02/12	MFC	-->	STL
//			:	2015/11/10	Add MessageBar
//			:	2016/04/25	remains of the array
//			:	2019/08/06	change attribute value quot
//	*******************************************************************************
inline	v_tstring	Xml_Out::CreateElement	(const Xml_E& elements,const int indent)
{
	v_tstring	strAry ;
	tstring		tab = _T("\t\t\t\t\t\t\t\t\t\t\t\t\t") ;
//	int			nowIndent = indent ;
	if (elements.GetName().empty()) {
		#ifdef	_DEBUG
		strAry.push_back(tab.substr(0,indent) + _T("<!--")+elements.GetText().c_str()+_T("-->")) ;
		#endif
		return	strAry ;
		}
	if (elements.IsComment()) {
		tstring	cmtStr;
		{
			cmtStr +=		tab.substr(0,indent) + _T("<!--") +	elements.GetText().c_str()	+	_T("-->") ;
			if (elements.GetChildCount() > 0) {
				cmtStr +=	_T("\r\n") ;
				cmtStr +=	tab.substr(0,indent) +		 _T("<!--") ;
				{
					v_tstring	saChild ;
					for (int cIndex=0 ; cIndex<elements.GetChildCount() ; cIndex++) {
						Xml_E		c = elements.GetChild(cIndex) ;
						v_tstring	sac = CreateElement(c,indent+1) ;
						saChild.insert(saChild.end(),sac.begin(),sac.end()) ;
						}
					if (saChild.size() > 0) {
						cmtStr += _T("\r\n") ;
						tstring	strChild = ::String_Join_Line(saChild) ;
						cmtStr += strChild ;
						}
					}
				cmtStr +=	tab.substr(0,indent+1) 		+	_T("-->") ;
				}
			}
		strAry.push_back(cmtStr) ;
		return	strAry ;
		}
	{											//	Start tag
		{
			tstring	eleStr = tab.substr(0,indent) + _T("<") + elements.GetName().c_str() + _T("\t") ;
			tstring	attStr ;						//	Attribute
			for (int aIndex=0 ; aIndex<elements.GetAttributeCount() ; aIndex++) {
				Xml_Attribute	a = elements.GetAttribute(aIndex) ;
				tstring	name = a.GetName ().c_str() ;
				tstring	value= a.GetValue().c_str() ;
			//													attStr += name + _T("=\"") + value + _T("\"\t") ;
				if (value.find(_T('\"')) != tstring::npos)	{	attStr += name + _T("=\'") + value + _T("\'\t") ;	}
				else										{	attStr += name + _T("=\"") + value + _T("\"\t") ;	}
				}
			if (!attStr.empty()) {
				eleStr += attStr ;
				}
			if (elements.GetText().empty() && elements.GetChildCount()==0) {	//	child , text  is empty ?
				eleStr += _T("/>") ;				//	<tag attr1='1' ... attrn='n' />
				strAry.push_back(eleStr) ;			eleStr.resize(0) ;				//	-------------------------------------
				//	---------------------------------------------------------------------------------------------------------
				}
			else {
				eleStr += _T(">") ;
			//	bool	existChild = false ;
			//	bool	existText  = false ;
				tstring	nameOrChild ;
				if (!elements.GetText().empty()) {	//	text
					tstring	strText = elements.GetText().c_str() ;
					eleStr += strText ;
				//	existText = true ;
					}
				if (elements.GetChildCount() == 0)	{
					eleStr += 	tstring(_T("</")) + elements.GetName().c_str() + _T(">") ;
					strAry.push_back(eleStr) ;		eleStr.resize(0) ;				//	-------------------------------------
					//	-----------------------------------------------------------------------------------------------------
					}
				else {
					strAry.push_back(eleStr) ;		eleStr.resize(0) ;				//	-------------------------------------
					//	-----------------------------------------------------------------------------------------------------
					v_tstring	eleStrA ;
					v_tstring	saChild ;			//	child
					MessageBar	bar(_T("Xml_Out::CreateElement ..."),elements.GetChildCount()) ;
					for (int cIndex=0 ; cIndex<elements.GetChildCount() ; cIndex++) {
						bar.SetBarInc() ;
						Xml_E		c = elements.GetChild(cIndex) ;
						v_tstring	sac = CreateElement(c,indent+1) ;	//	Recursive Call
						saChild.insert(saChild.end(),sac.begin(),sac.end()) ;
						}
					if (saChild.size() > 0) {
						eleStrA.insert(eleStrA.end(),saChild.begin(),saChild.end()) ;
					//	existChild = true ;
						}
					{							//	<tag attr1='1' ... > ... </tag>
						eleStr = tab.substr(0,indent+1) +	_T("</") + elements.GetName().c_str() + _T(">") ;
						eleStrA.push_back(eleStr) ;	eleStr.resize(0) ;				//	-------------------------------------
						}
					strAry.insert(strAry.end(),eleStrA.begin(),eleStrA.end()) ;		//	-------------------------------------
					//	-----------------------------------------------------------------------------------------------------
					}
				}
			}
		}
	return	strAry ;
	}

////
//	*******************************************************************************
//	Add Comment
//	Create	:	2019/05/29
//	*******************************************************************************
inline	bool	Xml_Out::AddComment	(Xml_E& xml_e,c_v_tstring& cmt_ary)
{
	for (size_t index=0 ; index<cmt_ary.size() ; index++) {
		tstring	cmt1 = cmt_ary[index] ;
		Xml_E	c_e ;	c_e.SetComment(cmt1) ;
		xml_e.AddChild(c_e) ;
		}
	return	true ;
	}

////
//	*******************************************************************************
//	Add Comment
//	Create	:	2012/09/25
//	Modify	:	2014/02/12	MFC	-->	STL
//	Modify	:	2019/05/29
//	*******************************************************************************
inline	bool	Xml_Out::AddComment	(Xml_E& xml_e,LPCTSTR comment)
{
	v_tstring	cmt_ary = ::String_SplitLine(comment) ;
	return		AddComment(xml_e,cmt_ary) ;
	}

////
//	*******************************************************************************
//	Set Comment
//	Create	:	2012/10/31
//	Modify	:	2014/02/12	MFC	-->	STL
//	*******************************************************************************
inline	bool	Xml_Out::SetComment	(Xml_E* xml_e,LPCTSTR comment)
{
	#ifdef	_DEBUG
		return	Xml_Out::SetCommentTree	(xml_e,	comment) ;
	#else
		Xml_E	xml_c ;		//	Delete child
				xml_c.SetComment		(		comment) ;
		*xml_e = xml_c ;
		return	true ;
	#endif
	}

inline	bool	Xml_Out::SetCommentTree	(Xml_E* xml_e,LPCTSTR comment)
{
	Xml_E	xml_cmt ;
	xml_cmt.SetComment(comment) ;
	xml_cmt.AddChild(*xml_e) ;
	*xml_e = xml_cmt ;
	return	true ;
	}

