//	*******************************************************************************
//	Name	:	FVersion.hxx
//			:	File version
//	Author	:	I. Nakagawa
//	Create	:	2022/07/27
//	Modify	:	2023/04/27
//	Modify	:	2024/08/09
//	*******************************************************************************

#pragma		once

#pragma		comment	(lib, "Version.lib")

#include	<Windows.h>

#include	"tstring.hxx"
#include	"i_v_type.hxx"
#include	"existff.hxx"
#include	"filepath.hxx"
#include	"stringto.hxx"
#include	"get_path.hxx"

//#ifdef	_DEBUG
//#include	"timefmt.hxx"
//#include	"vchrfile.hxx"
//#endif

/*
	VS_VERSION_INFO VERSIONINFO
	 FILEVERSION 1,53,2022,1
	 PRODUCTVERSION 1,53,2022,1
	 FILEFLAGSMASK 0x3fL
	#ifdef _DEBUG
	 FILEFLAGS 0x1L
	#else
	 FILEFLAGS 0x0L
	#endif
	 FILEOS 0x4L
	 FILETYPE 0x1L
	 FILESUBTYPE 0x0L
	BEGIN
	    BLOCK "StringFileInfo"
	    BEGIN
	        BLOCK "041104b0"
	        BEGIN
	            VALUE "Comments", "\0"
	            VALUE "CompanyName", "\0"
	            VALUE "FileDescription", "3D Viewer\0"
	            VALUE "FileVersion", "1, 53, 2022, 1\0"
	            VALUE "InternalName", "i3DV\0"
	            VALUE "LegalCopyright", "Copyright (C) 2013-2022  I. Nakagawa\0"
	            VALUE "LegalTrademarks", "\0"
	            VALUE "OriginalFilename", "i3DV.exe\0"
	            VALUE "PrivateBuild", "\0"
	            VALUE "ProductName", "i_Tools\0"
	            VALUE "ProductVersion", "1, 53, 2022, 1\0"
	            VALUE "SpecialBuild", "\0"
	        END
	    END
	    BLOCK "VarFileInfo"
	    BEGIN
	        VALUE "Translation", 0x411, 1200
	    END
	END
	*/

////
//	*******************************************************************************
//	File version
//	Create	:	2022/07/27
//	*******************************************************************************
class	FVersion	{
public:
					FVersion			(LPCTSTR    file=tstring().c_str()) ;
//	virtual			~FVersion			() ;
public:
	virtual	bool	SetFile				(LPCTSTR    file) ;
	virtual	bool	SetFile				(c_tstring& file)	{	return	SetFile(file.c_str()) ;		}
public:
	virtual	bool	GetInfo				(void) ;
	virtual	tstring	GetInfo				(LPCTSTR name) ;
public:
	virtual	tstring	GetComments			(void)	{	return	GetInfo(_T("Comments")) ;			}
	virtual	tstring	GetCompanyName		(void)	{	return	GetInfo(_T("CompanyName")) ;		}
	virtual	tstring	GetFileDescription	(void)	{	return	GetInfo(_T("FileDescription")) ;	}
	virtual	tstring	GetFileVersion		(void)	{	return	GetInfo(_T("FileVersion")) ;		}
	virtual	tstring	GetInternalName		(void)	{	return	GetInfo(_T("InternalName")) ;		}
	virtual	tstring	GetLegalCopyright	(void)	{	return	GetInfo(_T("LegalCopyright")) ;		}
	virtual	tstring	GetLegalTrademarks	(void)	{	return	GetInfo(_T("LegalTrademarks")) ;	}
	virtual	tstring	GetOriginalFilename	(void)	{	return	GetInfo(_T("OriginalFilename")) ;	}
	virtual	tstring	GetPrivateBuild		(void)	{	return	GetInfo(_T("PrivateBuild")) ;		}
	virtual	tstring	GetProductName		(void)	{	return	GetInfo(_T("ProductName")) ;		}
	virtual	tstring	GetProductVersion	(void)	{	return	GetInfo(_T("ProductVersion")) ;		}
	virtual	tstring	GetSpecialBuild		(void)	{	return	GetInfo(_T("SpecialBuild")) ;		}
protected:
	tstring		Target ;
protected:
	DWORD		VerHandle ;
	DWORD		VerInfoSize ;
	v_char		VerData ;
	tstring		StringFileInfo ;
	} ;

////
//	*******************************************************************************
//	constructor
//	Create	:	2022/07/27
//	*******************************************************************************
inline	FVersion::FVersion(LPCTSTR file)
{
	VerHandle   = 0 ;
	VerInfoSize = 0 ;
	SetFile(file) ;
	}

////
//	*******************************************************************************
//	set file name
//	Create	:	2022/07/27
//	*******************************************************************************
inline	bool	FVersion::SetFile	(LPCTSTR file)
{
	Target = file ;
	if (Target.empty())				{	return	false ;		}
	if (::File_IsNothing(Target))	{	return	false ;		}
	return	GetInfo() ;
	}

////
//	*******************************************************************************
//	call GetFileVersionInfo
//	Create	:	2022/07/27
//	*******************************************************************************
inline	bool	FVersion::GetInfo	(void)
{
	VerHandle   = 0 ;
	VerInfoSize = 0 ;
	VerInfoSize = ::GetFileVersionInfoSize(LPTSTR(Target.c_str()),&VerHandle) ;
	if (VerInfoSize == 0) {
		std::terr << _T("::GetFileVersionInfoSize ... 0")	<<	std::endl ;
		return	false ;
		}
	VerData.resize(VerInfoSize*2) ;
	if (!::GetFileVersionInfo(LPTSTR(Target.c_str()),VerHandle,VerInfoSize,&VerData[0])) {
		std::terr << _T("::GetFileVersionInfo ... false")	<<	std::endl ;
		return	false ;
		}
	/*
	#ifdef	_DEBUG
	{
		tstring	tmp_path = ::GetTmpPath() ;
				tmp_path = ::Path_AddLastSP(tmp_path) + _T("i_Tools.tmp") ;
				tmp_path = ::Path_AddLastSP(tmp_path) + _T("FVersion") ;
				tmp_path = ::Path_AddLastSP(tmp_path) + ::Now_Format(_T("%Y%m%d")) ;
		tstring	tmp_name = ::Path_AddLastSP(tmp_path) + ::Path_GetName(Target) + _T(".dmp") ;
		if (::Folder_Create(tmp_path)) {
			::v_c_Save(tmp_name.c_str(),VerData) ;
			}
		}
	#endif
	*/
	{
		struct	LANGANDCODEPAGE	{
			WORD	wLanguage ;
			WORD	wCodePage ;
			}	*lpTranslate	= NULL ;
		UINT	lenT = 0 ;
		if (!::VerQueryValue(&VerData[0],_T("\\VarFileInfo\\Translation"),(void**)&lpTranslate,&lenT)) {
			std::terr << _T("::VerQueryValue ... false")	<<	std::endl ;
			return	false ;
			}
		if (lenT == 0) {
			std::terr << _T("::VerQueryValue   len = 0")	<<	std::endl ;
			return	false ;
			}
		tstring	lang_cp_str =	::u32to0t(lpTranslate->wLanguage,16,4) +
								::u32to0t(lpTranslate->wCodePage,16,4) ;
		StringFileInfo = _T("\\StringFileInfo\\") + lang_cp_str + _T("\\") ;
		}
	return	true ;
	}

////
//	*******************************************************************************
//	call VerQueryValue
//	Create	:	2022/07/28
//	*******************************************************************************
inline	tstring	FVersion::GetInfo	(LPCTSTR name)
{
	tstring	ver_info ;
	if (StringFileInfo.empty()) {
		if (!GetInfo()) {
			return	ver_info ;
			}
		}
	{
		UINT	lenFV = 0 ;
		TCHAR*	lpFV  = NULL ;
		tstring	info_name = StringFileInfo+name ;
		if (!::VerQueryValue(&VerData[0],LPTSTR(info_name.c_str()),(void**)&lpFV,&lenFV)) {
			std::terr << _T("::VerQueryValue ...   ")	<<	name	<<	std::endl ;
			return	ver_info;
			}
		if (lenFV == 0) {
			std::terr << _T("::VerQueryValue len=0 ")	<<	name	<<	std::endl ;
			return	ver_info ;
			}
		ver_info = lpFV ;
		}
	return	ver_info ;
	}

////
//	*******************************************************************************
//	FileVersion , ProductVersion , ...
//	Create	:	2022/08/01
//	*******************************************************************************
inline	tstring	Get_OriginalFilename(LPCTSTR exe_path)	{	FVersion	fv(exe_path) ;	return	fv.GetOriginalFilename() ;	}
inline	tstring	Get_FileVersion		(LPCTSTR exe_path)	{	FVersion	fv(exe_path) ;	return	fv.GetFileVersion     () ;	}
inline	tstring	Get_ProductVersion	(LPCTSTR exe_path)	{	FVersion	fv(exe_path) ;	return	fv.GetProductVersion  () ;	}

////
//	*******************************************************************************
//	get self OriginalFilename
//	Create	:	2022/08/05
//	Modify	:	2023/04/27	add _OPENMP
//	*******************************************************************************
inline	tstring	Get_Self_Original	(void)
{
	static		bool	Yet			=	true ;
	static		tstring	Self_original ;
	if (Yet) {
		#ifdef	_OPENMP
			#pragma	omp	critical		(_Get_Self_Original_)
		#endif
		{
			tstring		self_name = ::Get_module_name() ;
			Self_original = Get_OriginalFilename(self_name.c_str()) ;
			if (Self_original.empty()) {
				Self_original = ::Path_GetName(self_name) ;
				}
			Yet = false ;
			}
		}
	return	Self_original ;
	}

/*
#ifdef		_________2023_04_26_____________________________
inline	tstring	Get_Self_Original	(void)
{
	#ifdef	_OPENMP
	//	#pragma	omp	critical		(_Get_Self_Original_)
	#endif
	{
	#ifdef	_OPENMP
				bool	Yet			=	true ;
				tstring	Self_original ;
	#else
		static	bool	Yet			=	true ;
		static	tstring	Self_original ;
	#endif
		if (Yet) {
			tstring		self_name = ::Get_module_name() ;
			Self_original = Get_OriginalFilename(self_name.c_str()) ;
			if (Self_original.empty()) {
				Self_original = ::Path_GetName(self_name) ;
				}
			Yet = false ;
			}
		return	Self_original ;
		}
	}
#endif	//	_________2023_04_26_____________________________
*/

