//	**************************************************************************************************
//	Name	:	delff_e.hxx
//			:	delete file or folder at exit
//	Author	:	I. Nakagawa
//	Create	:	2014/12/17
//	Modify	:	2014/12/22
//	Modify	:	2015/01/15
//	**************************************************************************************************

#pragma		once

#include	"_tdefine.hxx"
#include	"filecopy.hxx"
#include	"enumfile.hxx"

////
//	*******************************************************************************
//	Delete File at exit
//	Create	:	2014/12/17
//	Modify	:	2015/01/15
//	Modify	:	2016/05/02	Add "Add(c_tstring)"
//	*******************************************************************************
class	DelFF_E		{
public:
					DelFF_E		()		{	;						}
	virtual			~DelFF_E	()		{	Copy() ;	Remove() ;	}
public:
	virtual	bool	Add			(c_v_tstring delFiles)	{	Del_Files.insert(Del_Files.end(),
															delFiles.begin(),delFiles.end()) ;	return	true ;	}
	virtual	bool	Add			(LPCTSTR delFile)		{	Del_Files.push_back(delFile) ;		return	true ;	}
	virtual	bool	AddMove		(LPCTSTR srcName,LPCTSTR dstName)	{
									Del_Files.push_back(srcName) ;
									size_t	dfCount = Del_Files.size() ;
									DestFiles.resize(dfCount,_T("")) ;
									DestFiles[dfCount-1] = dstName ;
									return	true ;
									}
	virtual	bool	Add			(c_tstring& delFile)	{	return	Add(delFile.c_str()) ;	}
public:
	virtual	bool	Copy		(void)					{
									for (size_t index=0 ; index<Del_Files.size() && index<DestFiles.size() ; index++) {
										tstring	delFile = Del_Files[index] ;
										tstring	dstFile = DestFiles[index] ;
										if (delFile.empty())										{	continue ;	}
										if (dstFile.empty())										{	continue ;	}
										if (::File_IsNothing(delFile))								{	continue ;	}
									//	if (::CopyFile	(delFile,dstFile,FALSE))					{	continue ;	}
									//	if (::CopyFile	(delFile.c_str(),dstFile.c_str(),FALSE))	{	continue ;	}
										if (::File_Copy	(delFile.c_str(),dstFile.c_str(),FALSE))	{	continue ;	}
										Del_Files[index] = _T("") ;		//	clear if can not copy
										}
									DestFiles.clear() ;
									return	true ;
									}
	virtual	bool	Remove		(void)					{
		Del_Files = ::Unique	(Del_Files) ;	//	Delete same name
		Del_Files = ::Sort		(Del_Files) ;	//	Sort
		Del_Files = ::Reverse	(Del_Files) ;	//	Reverse
		v_tstring	lateDelFolders ;
	{	//	delete file and empty folder
		for (size_t index=0 ; index<Del_Files.size() ; index++) {
			tstring	delFile = Del_Files[index] ;
			if (delFile.empty())	{	continue ;	}
			delFile = ::Path_DelLastSP(delFile) ;
			Del_Files[index] = _T("") ;
			if (::File_IsExist(delFile)) {
				if (File_IsDirectory(delFile.c_str())) {
					tstring	delFold = delFile ;
					if (::Folder_IsEmpty(delFold.c_str())) {
						::_trmdir(delFold.c_str()) ;
						}
					else {	//	delete folder later
						lateDelFolders.push_back(delFold) ;
						}
					continue ;
					}
				{	//	Delete File
					_tremove(delFile.c_str()) ;
					}
				}
			}
		}
	Del_Files.clear() ;
	return	true ;
	}
protected:
	v_tstring	Del_Files ;		//	Delete files
	v_tstring	DestFiles ;		//	Move Destination
	} ;

////
//	*******************************************************************************
//	Delete File at app exit
//	Create	:	2015/02/27
//	Modify	:	2017/06/27
//	*******************************************************************************
inline	bool	DelFF_AE_Add	(LPCTSTR 		delFile)	{
	bool	result = false ;
	#ifdef	_OPENMP
		#pragma	omp	critical	(DelFF_AE_Add)
	#endif
	{
		static	DelFF_E		DFE ;
		result = DFE.Add(delFile) ;
		}
	return	result ;
	}

////
//	*******************************************************************************
//	Delete File at app exit
//	Create	:	2015/02/27
//	*******************************************************************************
inline	bool	DelFF_AE_Add	(c_tstring&		delFile)	{	return	::DelFF_AE_Add(delFile.c_str()) ;	}
inline	bool	DelFF_AE_Add	(c_v_tstring&	delFiles)	{
	for (size_t index=0 ; index<delFiles.size() ; index++){
		::DelFF_AE_Add(delFiles[index]) ;
		}
	return	true ;
	}

////
//	*******************************************************************************
//	delete empty folder at app exit
//	Create	:	2017/01/18
//	Modify	:	2021/02/08	add "Thumbs.db"
//	*******************************************************************************
inline	bool	DelFF_AE_EmptyFolder	(LPCTSTR 	delFold)	{
	v_tstring	emptyFold = ::EnumFoldersTree(delFold) ;
#ifdef		_MSC_VER
	v_tstring	thmdbFile ;
	{
		for (size_t index=0 ; index<emptyFold.size() ; index++) {
			tstring	thmdb = ::Path_AddLastSP(emptyFold[index]) + _T("Thumbs.db") ;
			if (::File_IsExist(thmdb)) {
				thmdbFile.push_back(thmdb) ;
				}
			}
		}
	::DelFF_AE_Add(thmdbFile) ;
#endif
	::DelFF_AE_Add(emptyFold) ;
	::DelFF_AE_Add(delFold) ;
	return	true ;
	}

////
//	*******************************************************************************
//	delete files at app exit
//	Create	:	2019/02/26
//	*******************************************************************************
inline	bool	DelFF_AE_FilesTree		(LPCTSTR	delFold)
{
	v_tstring	del_files = ::EnumFilesTree(delFold) ;
				del_files.push_back(delFold) ;
	return	::DelFF_AE_Add(del_files) ;
	}

inline	bool	DelFF_AE_EmptyFolder	(c_tstring&	delFold)	{	return	::DelFF_AE_EmptyFolder(delFold.c_str()) ;	}
inline	bool	DelFF_AE_FilesTree		(c_tstring&	delFold)	{	return	::DelFF_AE_FilesTree  (delFold.c_str()) ;	}

