//	*******************************************************************************
//	Name	:	parse_tm.hxx
//			:	Parse Time
//	Author	:	I. Nakagawa
//	Create	:	2021/07/07
//	Modify	:	2021/07/07
//	*******************************************************************************

#pragma		once

#include	"stringto.hxx"
#include	"ttox.hxx"
#include	"timefmt.hxx"

////
//	*******************************************************************************
//	Parse Time
//	Create	:	2021/07/07
//	*******************************************************************************
inline	struct tm	Parse_time	(
	LPCTSTR		dtStr						//	"Thu Nov 28 00:36:10 2014"		"Nov 28 2014"		   "11:10:30"
,	LPCTSTR		p_fmt	=	TF_abdHMSY		//	_T("%a %b %d %H:%M:%S %Y"))		_T("%b %d %Y")		_T("%H:%M:%S")
	)
{
	tstring		fmt = 	p_fmt ;
	struct	tm	ptm ;	memset(&ptm,0,sizeof(ptm)) ;
	{
		v_tstring	chk_ary = ::String_SplitSpace(dtStr) ;
		if (chk_ary.size() == 1) {
			v_tstring	hms_ary = ::String_Split(dtStr,false,_T(":")) ;
			if (hms_ary.size() == 3) {
				fmt = TF_HMS ;
				}
			}
		else if (chk_ary.size() == 3) {
				fmt = TF_bdY ;
			}
		}
	{
		v_tstring	dtAry = ::String_SplitSpace(dtStr) ;
		tstring		wDay ;
		tstring		mon ;
		tstring		day ;
		tstring		hms ;
		tstring		year ;
		tstring		hour ;
		tstring		minu ;
		tstring		seco ;
		if		(dtAry.size() == 3 || fmt == TF_bdY) {
			if (dtAry.size() >= 1)	{	mon  = dtAry[0] ;	}
			if (dtAry.size() >= 2)	{	day  = dtAry[1] ;	}
			if (dtAry.size() >= 3)	{	year = dtAry[2] ;	}
			}
		else if	(dtAry.size() == 5 || fmt == TF_abdHMSY) {
			if (dtAry.size() >= 1)	{	wDay = dtAry[0] ;	}
			if (dtAry.size() >= 2)	{	mon  = dtAry[1] ;	}
			if (dtAry.size() >= 3)	{	day  = dtAry[2] ;	}
			if (dtAry.size() >= 4)	{	hms  = dtAry[3] ;	}
			if (dtAry.size() >= 5)	{	year = dtAry[4] ;	}
			v_tstring	hmsAry = ::String_Split(hms,false,_T(":")) ;
			if (hmsAry.size() >= 1)	{	hour = hmsAry[0] ;	}
			if (hmsAry.size() >= 2)	{	minu = hmsAry[1] ;	}
			if (hmsAry.size() >= 3)	{	seco = hmsAry[2] ;	}
			}
		else if (dtAry.size() == 1) {
			v_tstring	hmsAry = ::String_Split(dtAry[0],false,_T(":")) ;
			if (hmsAry.size() >= 1)	{	hour = hmsAry[0] ;	}
			if (hmsAry.size() >= 2)	{	minu = hmsAry[1] ;	}
			if (hmsAry.size() >= 3)	{	seco = hmsAry[2] ;	}
			}
		if (!hour.empty())				{	ptm.tm_hour = ::ttou2(hour) ;		}
		if (!minu.empty())				{	ptm.tm_min  = ::ttou2(minu) ;		}
		if (!seco.empty())				{	ptm.tm_sec  = ::ttou2(seco) ;		}
		if (!year.empty())				{	ptm.tm_year = ::ttou2(year)-1900 ;	}
		if (!day .empty())				{	ptm.tm_mday = ::ttou2(day) ;		}
		if (!mon .empty())	{
			if		(mon == _T("Jan"))	{	ptm.tm_mon	=	 0 ;		}
			else if	(mon == _T("Feb"))	{	ptm.tm_mon	=	 1 ;		}
			else if	(mon == _T("Mar"))	{	ptm.tm_mon	=	 2 ;		}
			else if	(mon == _T("Apr"))	{	ptm.tm_mon	=	 3 ;		}
			else if	(mon == _T("May"))	{	ptm.tm_mon	=	 4 ;		}
			else if	(mon == _T("Jun"))	{	ptm.tm_mon	=	 5 ;		}
			else if	(mon == _T("Jul"))	{	ptm.tm_mon	=	 6 ;		}
			else if	(mon == _T("Aug"))	{	ptm.tm_mon	=	 7 ;		}
			else if	(mon == _T("Sep"))	{	ptm.tm_mon	=	 8 ;		}
			else if	(mon == _T("Oct"))	{	ptm.tm_mon	=	 9 ;		}
			else if	(mon == _T("Nov"))	{	ptm.tm_mon	=	10 ;		}
			else if	(mon == _T("Dec"))	{	ptm.tm_mon	=	11 ;		}
			}
		if (!wDay.empty())	{
			if		(wDay== _T("Sun"))	{	ptm.tm_wday	=	 0 ;		}
			else if	(wDay== _T("Mon"))	{	ptm.tm_wday	=	 1 ;		}
			else if	(wDay== _T("Tue"))	{	ptm.tm_wday	=	 2 ;		}
			else if	(wDay== _T("Wed"))	{	ptm.tm_wday	=	 3 ;		}
			else if	(wDay== _T("Thu"))	{	ptm.tm_wday	=	 4 ;		}
			else if	(wDay== _T("Fri"))	{	ptm.tm_wday	=	 5 ;		}
			else if	(wDay== _T("Sat"))	{	ptm.tm_wday	=	 6 ;		}
			}
		}
	return	ptm ;
	}

////
//	*******************************************************************************
//	Get Time String
//	Create	:	2014/11/27
//	Modify	:	2021/07/07
//	*******************************************************************************
inline	tstring	TS_to_Time_string	(
	LPCTSTR		dtStr						//	"Thu Nov 28 00:36:10 2014"
,	LPCTSTR		fmt		=	TF_Ymd_HMS		//	_T("%Y/%m/%d %H:%M:%S")
,	LPCTSTR		p_fmt	=	TF_abdHMSY		//	_T("%a %b %d %H:%M:%S %Y"))		_T("%b %d %Y")
	)
{
	struct	tm	stm = ::Parse_time(dtStr,p_fmt) ;
	tstring		str = ::Time_Format(stm,fmt) ;
	return	str ;
	}

////
//	*******************************************************************************
//	Parse Time	"Nov 28 2014"
//	Create	:	2014/11/28
//	*******************************************************************************
//inline	struct tm	ParseTime_bdY	(LPCTSTR dtStr)	{	return	::Parse_time(dtStr,TF_bdY) ;	}
////
//	*******************************************************************************
//	Get Time String
//	Create	:	2014/11/27
//	*******************************************************************************
//inline	tstring	TStoTimeYmd		(LPCTSTR dtStr)		{	return	::TS_to_Time_string(dtStr,_T("%Y.%m.%d")) ;	}
//inline	tstring	TStoTimeYm		(LPCTSTR dtStr)		{	return	::TS_to_Time_string(dtStr,_T("%Y.%m")) ;	}

