//	*******************************************************************************
//	Name	:	straryfn.hxx
//			:	vector<tstring>	Functions
//	Author	:	I. Nakagawa
//	Create	:	2013/01/30
//	Modify	:	2013/09/02
//	Modify	:	2014/01/23
//	Modify	:	2014/04/30
//	Modify	:	2016/05/16
//	Modify	:	2017/02/14	Add "String_Unique"
//	*******************************************************************************

#pragma		once

#include	"v_tstrng.hxx"
#include	<algorithm>
#include	<cstring>
#include	"i_define.hxx"
#include	"stringfn.hxx"

#ifndef		_tcspbrk
#define		_tcspbrk		strpbrk
#endif
#ifndef		_tcsstr
#define		_tcsstr			strstr
#endif
#ifndef		_tcschr
#define		_tcschr			strchr
#endif

////
//	*******************************************************************************
//	Is 7 bit ?	String Array
//	Create	:	2014/06/19
//	*******************************************************************************
inline	bool	String_IsOnly7bit	(c_v_tstring& strAry)
{
	for (size_t index=0 ; index<strAry.size() ; index++) {
		tstring	str = strAry[index] ;
		if (!::String_IsOnly7bit(str))	{
			return	false ;
			}
		}
	return	true ;
	}

////
//	*******************************************************************************
//	Skip Empty String
//	Create	:	2013/01/30
//	Modify	:	2013/02/22
//	*******************************************************************************
inline	std::vector<tstring>	SkipEmpty	(const std::vector<tstring>& srcAry)
{
	std::vector<tstring>	strAry ;
	for (size_t index=0 ; index<srcAry.size() ; index++) {
		tstring	str = srcAry[index] ;
		if (str.empty())	{	continue ;	}
		strAry.push_back(str) ;
		}
	return	strAry ;
	}

////
//	*******************************************************************************
//	Sort String
//	Create	:	2013/01/30
//	*******************************************************************************
inline	std::vector<tstring>	Sort	(const std::vector<tstring>& srcAry)
{
	std::vector<tstring>	strAry = srcAry ;
	std::sort(strAry.begin(),strAry.end()) ;
	return	strAry ;
	}

////
//	*******************************************************************************
//	Reverse String
//	Create	:	2013/09/03
//	*******************************************************************************
inline	std::vector<tstring>	Reverse	(const std::vector<tstring>& srcAry)
{
	std::vector<tstring>	strAry = srcAry ;
	std::reverse(strAry.begin(),strAry.end()) ;
	return	strAry ;
	}

////
//	*******************************************************************************
//	Unique String
//	Create	:	2013/01/30
//	*******************************************************************************
inline	std::vector<tstring>	Unique	(const std::vector<tstring>& srcAry)
{
	std::vector<tstring>	strAry = srcAry ;
	std::sort(strAry.begin(),strAry.end()) ;
	std::vector<tstring>::iterator	uni = std::unique(strAry.begin(),strAry.end()) ;
	strAry.erase(uni,strAry.end()) ;
	return	strAry ;
	}

////
//	*******************************************************************************
//	Unique String	non sort
//	Create	:	2017/02/07
//	*******************************************************************************
inline	std::vector<tstring>	String_Unique	(const std::vector<tstring>& srcAry)
{
	std::vector<tstring>	uniqueAry ;
	for (size_t index=0 ; index<srcAry.size() ; index++) {
		tstring	code = srcAry[index] ;
		bool	found = false ;
		for (size_t uIndex=0 ; uIndex<uniqueAry.size() ; uIndex++) {
			if (code == uniqueAry[uIndex]) {
				found = true ;
				break ;
				}
			}
		if (!found) {
			uniqueAry.push_back(code) ;
			}
		}
	return	uniqueAry ;
	}

////
//	*******************************************************************************
//	Unique String	non sort	ignore case
//	Create	:	2017/02/07
//	*******************************************************************************
inline	std::vector<tstring>	Unique_nocase	(const std::vector<tstring>& srcAry)
{
	std::vector<tstring>	uniqueAry ;
	for (size_t index=0 ; index<srcAry.size() ; index++) {
		tstring	code = srcAry[index] ;
		tstring	lowc = ::String_ToLower(code) ;
		bool	found = false ;
		for (size_t uIndex=0 ; uIndex<uniqueAry.size() ; uIndex++) {
			tstring	low_u1 = ::String_ToLower(uniqueAry[uIndex]) ;
			if (lowc == low_u1) {
				found = true ;
				break ;
				}
			}
		if (!found) {
			uniqueAry.push_back(code) ;
			}
		}
	return	uniqueAry ;
	}

////
//	*******************************************************************************
//	Split
//	Create	:	2013/03/06
//	Modify	:	2013/09/02
//	Modify	:	2014/01/14	add isLine	...	split "\r\n"
//	Modify	:	2014/04/30	Bug		isLine Last CrLf
//	Modify	:	2016/05/16	Bug		_tcspbrk ... was not _mbspbrk
//	*******************************************************************************
inline	std::vector<tstring>	String_Split	(const tstring& str_,const bool isLine,LPCTSTR sp=_T("\r\n"))
{
	bool		mb_1 = false ;				//	MBCS 1 Byte
	#ifdef	_UNICODE
	#else
		if (_tcslen(sp) == 1) {
			if (sp[0] == '\\') {				//	0x5c
				mb_1 = true ;
				}
			}
	#endif
	std::vector<tstring>	strAry ;
	tstring		str = str_ ;
	LPCTSTR		pTmp = str.c_str() ;
	LPTSTR		pStart = LPTSTR(pTmp) ;
	LPTSTR		pFind = NULL ;
	while	(true) {
		if (*pStart == '\"') {				//	"---"
			pFind = _tcspbrk(pStart+1,_T("\"")) ;
			if (pFind == NULL)	{	break ;		}
			pFind = _tcspbrk(pFind+1,sp) ;
			}
		else if (*pStart == '\'') {			//	'---'
			pFind = _tcspbrk(pStart+1,_T("\'")) ;
			if (pFind == NULL)	{	break ;		}
			pFind = _tcspbrk(pFind+1,sp) ;
			}
		else {
			if (isLine)		{	pFind = _tcsstr (pStart,sp) ;	}
			else if (mb_1)	{	pFind = _tcschr	(pStart,sp[0]);	}
			else			{	pFind = _tcspbrk(pStart,sp) ;	}
			}
		if (pFind == NULL)	{	break ;		}
		int	bufLen = int(pFind-pStart+10) ;
		std::vector<TCHAR>	tmpStr ;
		tmpStr.resize(bufLen,0) ;
		::TcsNCpy(&tmpStr[0],bufLen,pStart,pFind-pStart) ;
		tstring	field = &tmpStr[0] ;
		strAry.push_back(field) ;
		if (isLine)		{	pFind += _tcslen(sp) ;	}
		else			{	pFind++ ;				}
		pStart = pFind ;
		}
	if (_tcslen(pStart) > 0) {
		strAry.push_back(pStart) ;
		}
	return	strAry ;
	}
inline	std::vector<tstring>	String_Split	(const tstring& str,const tstring sp)
{
	return	::String_Split(str,false,sp.c_str()) ;
	}
inline	std::vector<tstring>	String_Split	(const tstring& str,const TCHAR sp/*=_T(' ')*/)
{
	tstring	spStr ;	spStr = sp ;
	return	::String_Split(str,false,spStr.c_str()) ;
	}

////
//	*******************************************************************************
//	Join
//	Create	:	2013/03/06
//	Modify	:	2013/12/24
//	Modify	:	2015/11/19	VC 6 - 8  recursive call	iwaoalles.wordpress.com
//	*******************************************************************************
#if(_MSC_VER >= 1500)		//	VC 9 -
inline	tstring	String_Join	(const std::vector<tstring>& srcAry,LPCTSTR sp)
{
	tstring	str ;
	#ifdef		MessageBar
	MessageBar	bar(_T("::String_Join ..."),srcAry.size()) ;
	#endif
	for (size_t index= 0 ; index<srcAry.size() ; index++) {
	#ifdef		MessageBar
		bar.SetBarInc() ;
	#endif
		str += srcAry[index] ;
		if (index+1 == srcAry.size())	{	continue ;	}
		str += sp ;
		}
	return	str ;
	}
#else						//	- VC 8
inline	tstring	String_Join	(const std::vector<tstring>& srcAry,LPCTSTR sp)
{
	tstring		str ;
	v_tstring	tmpSA ;
	tstring		tmpStr ;
	#ifdef		MessageBar
	MessageBar	bar(_T("::String_Join ..."),srcAry.size()) ;
	#endif
	for (size_t index= 0 ; index<srcAry.size() ; index++) {
	#ifdef		MessageBar
		bar.SetBarInc() ;
	#endif
		tmpStr += srcAry[index] ;
		if (index+1 == srcAry.size())	{	continue ;	}
		if ((index%128) == 100) {
			tmpSA.push_back(tmpStr) ;
			tmpStr.erase() ;
			continue ;
			}
		tmpStr += sp ;
		}
	if (!tmpStr.empty()) {
		tmpSA.push_back(tmpStr) ;
		}
	if (tmpSA.size() > 1) {
		str = ::String_Join(tmpSA,sp) ;
		}
	else if (tmpSA.size() == 1)	{
		str = tmpSA[0] ;
		}
	return	str ;
	}
#endif

inline	tstring	String_Join	(const std::vector<tstring>& srcAry,const tstring sp)
{
	return	::String_Join(srcAry,sp.c_str()) ;
	}
inline	tstring	String_Join	(const std::vector<tstring>& srcAry,const TCHAR sp=_T(' '))
{
	tstring	spStr ;	spStr = sp ;
	return	::String_Join(srcAry,spStr) ;
	}
