//	*******************************************************************************
//	Name	:	timefmt.hxx
//			:	Time Format
//	Author	:	I. Nakagawa
//	Create	:	2013/08/28
//	Modify	:	2013/11/27
//			:	2014/10/08
//			:	2014/11/27
//			:	2019/05/23
//	*******************************************************************************

#pragma		once

#include	<ctime>
#include	"i_define.hxx"
#include	"tstring.hxx"

#define		TF_Ymd_HMS		_T("%Y/%m/%d %H:%M:%S")
#define		TF_Ymd			_T("%Y/%m/%d")
#define		TF_HMS			_T("%H:%M:%S")
#define		TF_abdHMSY		_T("%a %b %d %H:%M:%S %Y")
#define		TF_bdY			_T("%b %d %Y")

#ifdef		_MSC_VER
#else
#define		_tcsftime	strftime
#endif

////
//	*******************************************************************************
//	struct tm	Fomat
//	Create	:	2014/11/27
//	*******************************************************************************
inline	tstring	Time_Format	(const struct tm& stm,LPCTSTR fmt=TF_Ymd_HMS)
{
	tstring	buff ;
	size_t	size = 255 ;
	buff.resize(size+1,0) ;
	if (::_tcsftime(&buff[0],size,fmt,&stm) == 0)	{	return	_T("") ;	}
	return	buff.c_str() ;
	}

////
//	*******************************************************************************
//	time_t	Fomat
//	Create	:	2013/08/28
//	Modify	:	2014/11/27
//	*******************************************************************************
inline	tstring	Time_Format	(const time_t& tim_v,LPCTSTR fmt=TF_Ymd_HMS)
{
	struct	tm*	ptm = NULL ;
#if	(_MSC_VER >= 1400)
	struct	tm	ptmTemp ;
	errno_t err = _localtime64_s(&ptmTemp,&tim_v) ;
	if (err != 0)									{	return	_T("") ;	}
	ptm = &ptmTemp ;
#else
	ptm = ::localtime(&tim_v) ;
	if (ptm == NULL)								{	return	_T("") ;	}
#endif
	return	::Time_Format(*ptm,fmt) ;
	}

////
//	*******************************************************************************
//	Now
//	Create	:	2013/11/27
//	*******************************************************************************
inline	tstring	Now_Format	(LPCTSTR fmt=TF_Ymd_HMS)
{
	return	::Time_Format(::time(NULL),fmt) ;
	}

////
//	*******************************************************************************
//	time_t	Fomat GMT
//	VS/VC98/MFC/SRC/TIMECORE.CPP
//	VS8/VC/atlmfc/include/atltime.h
//	Create	:	2019/05/23
//	*******************************************************************************
inline	tstring	Time_FormatGMT	(const time_t& tim_v,LPCTSTR fmt=TF_Ymd_HMS)
{
	struct	tm*	ptm = NULL ;
#if	(_MSC_VER >= 1400)
	struct	tm	ptmTemp ;
	errno_t err = _gmtime64_s(&ptmTemp,&tim_v) ;
	if (err != 0)									{	return	_T("") ;	}
	ptm = &ptmTemp ;
#else
	ptm = ::gmtime(&tim_v) ;
	if (ptm == NULL)								{	return	_T("") ;	}
#endif
	return	::Time_Format(*ptm,fmt) ;
	}

////
//	*******************************************************************************
//	Now GMT
//	Create	:	2019/05/23
//	*******************************************************************************
inline	tstring	Now_FormatGMT	(LPCTSTR fmt=TF_Ymd_HMS)
{
	return	::Time_FormatGMT(::time(NULL),fmt) ;
	}

