//	*******************************************************************************
//	Name	:	gettickc.hxx
//			:	Get Tick Count
//	Author	:	I. Nakagawa
//	Create	:	2013/08/27
//	*******************************************************************************

#pragma		once

#include		<cstring>
#include		<cerrno>
#include		"i_define.hxx"
#include		"i_debug.hxx"

#ifdef	_WIN32
	#include	<Windows.h>
#endif
#ifdef	__GNUC__
	#include	<sys/time.h>
	#include	<unistd.h>
#endif

////
//	*******************************************************************************
//	Get Tick Count
//	Create	:	2013/08/27
//	*******************************************************************************
inline	double	GetTickCount_s	(void)
{
	double	tval_s = 0 ;
	#ifdef	_WIN32
		tval_s = double(::GetTickCount())*1e-3 ;
	#endif
	#ifdef	__GNUC__
	{
		static	time_t	tc_1st = 0 ;
	//	if (tc_1st == 0)	{
	//		tc_1st = ::time(NULL) ;
	//		}
		struct	timeval	tv ;	memset(&tv,0,sizeof(timeval)) ;
		if (gettimeofday(&tv,NULL) != 0)	{	::i_Dump(errno) ;		return	0 ;		}
		tval_s = double(tv.tv_sec-tc_1st) + double(tv.tv_usec)*1e-6 ;
		}
	#endif
	return	tval_s ;
	}
/*
inline	i_64	GetTickCount_ms	(void)
{
	i_64	tick = 0 ;
	#ifdef	_WIN32
		tick = ::GetTickCount() ;
	#endif
	#ifdef	__GNUC__
		{
			double	sec = ::GetTickCount_s() ;
			tick = int(sec*1000) ;
			}
	#endif
	return	tick ;
	}
*/

////
//	*******************************************************************************
//	Sleep
//	Create	:	2013/08/27
//	*******************************************************************************
inline	bool	Sleep_ms	(const u_32 msec)
{
	#ifdef	_WIN32
		::Sleep(msec) ;
	#endif
	#ifdef	__GNUC__
		if (::usleep(msec*1000) != 0)	{	::i_Dump(errno) ;		return	false ;		}
	#endif
	return	true ;
	}

