//	**************************************************************************************************
//	Name	:	htmout.hxx
//			:	HTML out
//	Author	:	I. Nakagawa
//	Create	:	2015/10/16
//	Modify	:	2016/12/07
//	Modify	:	2017/01/26	add	img_b ...
//	Modify	:	2019/02/08	add meta viewport
//	**************************************************************************************************

#pragma		once

#include	"xml_base.hxx"
#include	"xml_css.hxx"
#include	"xml_out.hxx"
#include	"parse_tm.hxx"
#include	"itls_tmp.hxx"

#define	HTM_html		_T("html")
#define	HTM_lang		_T("lang")
#define	HTM_head		_T("head")
#define	HTM_meta		_T("meta")
#define	HTM_charset		_T("charset")
#define	HTM_name		_T("name")
#define	HTM_content		_T("content")
#define	HTM_title		_T("title")

#define	HTM_script		_T("script")
#define	HTM_link		_T("link")
#define	HTM_type		_T("type")
#define	HTM_rel			_T("rel")
#define	HTM_style		_T("style")
#define	HTM_javascript	_T("text/javascript")
#define	HTM_stylesheet	_T("stylesheet")

#define	HTM_class		_T("class")
#define	HTM_id			_T("id")

#define	HTM_body		_T("body")
#define	HTM_br			_T("br")
#define	HTM_hr			_T("hr")
#define	HTM_pre			_T("pre")
#define	HTM_span		_T("span")
#define	HTM_div			_T("div")

#define	HTM_a			_T("a")
#define	HTM_href		_T("href")

#define	HTM_img			_T("img")
#define	HTM_src			_T("src")
#define	HTM_alt			_T("alt")

#define	HTM_table		_T("table")
#define	HTM_border		_T("border")
#define	HTM_tr			_T("tr")
#define	HTM_th			_T("th")
#define	HTM_td			_T("td")

#define	HTM_ul			_T("ul")
#define	HTM_li			_T("li")

#define	HTM_width		_T("width")
#define	HTM_align		_T("align")
#define	HTM_right		_T("right")
#define	HTM_center		_T("center")
#define	HTM_valign		_T("valign")
#define	HTM_top			_T("top")

class	HtmOut		{
public:
	////
	//	*******************************************************************************
	//	html
	//	Create	:	2015/10/16
	//	*******************************************************************************
	static	Xml_E	html		(void)	{
			Xml_E	htm(HTM_html) ;
					htm.AddAttribute(HTM_lang,_T("ja")) ;
			return	htm ;
			}

	////
	//	*******************************************************************************
	//	meta charset
	//	Create	:	2015/10/22
	//	*******************************************************************************
	static	Xml_E	charset_UTF_8	(void)	{
			Xml_E	meta(HTM_meta) ;
					meta.AddAttribute(HTM_charset,_T("UTF-8")) ;
			return	meta ;
			}

	////
	//	*******************************************************************************
	//	meta viewport
	//	Create	:	2019/02/08
	//	*******************************************************************************
	static	Xml_E	meta_viewport	(void)	{
			Xml_E	meta(HTM_meta) ;
					meta.AddAttribute(HTM_name,		_T("viewport")) ;
					meta.AddAttribute(HTM_content,	_T("width=device-width,initial-scale=1.0")) ;
			return	meta ;
			}

	////
	//	*******************************************************************************
	//	title
	//	Create	:	2015/10/22
	//	Modify	:	2017/01/20	check empty
	//	*******************************************************************************
	static	Xml_E	title		(c_tstring& name)	{
			Xml_E	ttl ;
			if (!name.empty()) {
					ttl.SetName(HTM_title) ;
					ttl.SetText(name) ;
					}
			return	ttl ;
			}

	////
	//	*******************************************************************************
	//	script
	//	Create	:	2017/02/08
	//	*******************************************************************************
	static	Xml_E	script		(c_tstring&type,c_tstring& src)	{
			Xml_E	scr(HTM_script) ;
					scr.AddAttribute(HTM_type,type) ;
			if (!src.empty()) {
					scr.AddAttribute(HTM_src, src) ;
					}
					scr.SetText(_T("\t")) ;
			return	scr ;
			}
	static	Xml_E	script		(				c_tstring& src=_T(""))	{
			Xml_E	scr = script(HTM_javascript,src) ;
			return	scr ;
			}

	////
	//	*******************************************************************************
	//	link
	//	Create	:	2017/02/08
	//	*******************************************************************************
	static	Xml_E	link		(c_tstring& rel,c_tstring& ref)	{
			Xml_E	lnk(HTM_link) ;
					lnk.AddAttribute(HTM_rel, rel) ;
					lnk.AddAttribute(HTM_href,ref) ;
			return	lnk ;
			}
	static	Xml_E	link_ss		(				c_tstring& ref)	{
			Xml_E	lnk = link(HTM_stylesheet,ref) ;
			return	lnk ;
			}

	////
	//	*******************************************************************************
	//	style
	//	Create	:	2017/02/07
	//	*******************************************************************************
	static	Xml_E	style		(c_tstring& data)	{
			Xml_E	sty(HTM_style) ;
					sty.SetText(data) ;
			return	sty ;
			}

	////
	//	*******************************************************************************
	//	comment
	//	Create	:	2016/11/15
	//	*******************************************************************************
	static	Xml_E	comment		(c_tstring& cmmnt)	{
			Xml_E	cmt ;
					cmt.SetComment(cmmnt) ;
			return	cmt ;
			}

	static	Xml_E	comment		(void)	{
			tstring	cmmnt ;
				{
					tstring	expStr = ::TS_to_Time_string(_T(__TIMESTAMP__))	+						_T("\t") + ::Path_GetName(_T(__FILE__)) ;
					tstring	exeStr = ::TS_to_Time_string(_T(__DATE__),TF_Ymd)+_T(" ") _T(__TIME__) 	_T("\t") ;
							exeStr+= ::Get_module_name() ;
					cmmnt +=														_T("\r\n") ;
					cmmnt += _T("HTML Exporter\t1.02\t")	+	expStr			+	_T("\r\n") ;
					cmmnt += _T("EXE          \t    \t")	+	exeStr			+	_T("\r\n") ;
					cmmnt += _T("Time         \t    \t")	+	::Now_Format()	+	_T("\r\n") ;
				//	cmmnt += _T("iwaoalles.wordpress.com\t\tiwaon@hotmail.com")		_T("\r\n") ;
					cmmnt += _T("itl.mydns.jp \t    \t")  _T("iwaon@hotmail.com")	_T("\r\n") ;
				//	cmmnt += _T("iwao.synology.me\t \t")  _T("iwaon@hotmail.com")	_T("\r\n") ;
					}
			return	comment(cmmnt) ;
			}

	////
	//	*******************************************************************************
	//	br
	//	Create	:	2015/10/22
	//	*******************************************************************************
	static	Xml_E	br			(void)	{
			Xml_E	br_(HTM_br) ;
			return	br_ ;
			}

	////
	//	*******************************************************************************
	//	hr
	//	Create	:	2016/12/07
	//	*******************************************************************************
	static	Xml_E	hr			(void)	{
			Xml_E	hr_(HTM_hr) ;
			return	hr_ ;
			}

	////
	//	*******************************************************************************
	//	span
	//	Create	:	2015/10/22
	//	*******************************************************************************
	static	Xml_E	span		(c_tstring& text)	{
			Xml_E	spn(HTM_span) ;
					spn.SetText(text) ;
			return	spn ;
			}

	////
	//	*******************************************************************************
	//	div
	//	Create	:	2016/12/07
	//	Modify	:	2017/05/25
	//	*******************************************************************************
	static	Xml_E	div			(void)	{
			Xml_E	dv_(HTM_div) ;
			return	dv_ ;
			}
	static	Xml_E	div			(c_tstring& text)	{
			Xml_E	dv_ = div() ;
					dv_.SetText(text) ;
			return	dv_ ;
			}
	static	Xml_E	div_c		(void)	{
			Xml_E	dv_(HTM_div) ;
			//		dv_.AddAttribute(HTM_align,HTM_center) ;
					dv_.AddAttribute(HTM_style,CSS_text_align_center()) ;
			return	dv_ ;
			}
	static	Xml_E	div_r		(void)	{
			Xml_E	dv_(HTM_div) ;
			//		dv_.AddAttribute(HTM_align,HTM_right) ;
					dv_.AddAttribute(HTM_style,CSS_text_align_right()) ;
			return	dv_ ;
			}
	static	Xml_E	div_c		(c_tstring& text)	{
			Xml_E	dv_ = div_c() ;
					dv_.SetText(text) ;
			return	dv_ ;
			}
	static	Xml_E	div_r		(c_tstring& text)	{
			Xml_E	dv_ = div_r() ;
					dv_.SetText(text) ;
			return	dv_ ;
			}

	////
	//	*******************************************************************************
	//	pre
	//	Create	:	2016/11/15
	//	*******************************************************************************
	static	Xml_E	pre			(c_tstring& text)	{
			Xml_E	pr_(HTM_pre) ;
					pr_.SetText(text) ;
			return	pr_ ;
			}

	////
	//	*******************************************************************************
	//	a
	//	Create	:	2015/10/22
	//	*******************************************************************************
	static	Xml_E	a			(c_tstring& ref,c_tstring& text)	{
			Xml_E	a_(HTM_a) ;
					a_.AddAttribute(HTM_href,ref) ;
					a_.SetText(text) ;
			if (text.empty()) {
					a_.SetText(_T(" ")) ;
					}
			return	a_ ;
			}

	static	Xml_E	a			(c_tstring& name)	{	return	a(name,name) ;							}
	static	Xml_E	a_current		(void)			{	return	a(_T("."),				_T("./")) ;		}
	static	Xml_E	a_current_index	(void)			{	return	a(_T("./index.htm"),	_T(".")) ;		}
	static	Xml_E	a_parent		(void)			{	return	a(_T(".."),				_T("../")) ;	}
	static	Xml_E	a_parent_index	(void)			{	return	a(_T("../index.htm"),	_T("..")) ;		}

	////
	//	*******************************************************************************
	//	img
	//	Create	:	2016/11/18
	//	Modify	:	2017/01/26	Add	img_b ...
	//	Modify	:	2017/02/06	Add	img class ...
	//	*******************************************************************************
	static	Xml_E	img			(c_tstring& cName,c_tstring& src)	{
			Xml_E	img_(HTM_img) ;
			if (!cName.empty()) {
					img_.AddAttribute(HTM_class,cName) ;
					}
					img_.AddAttribute(HTM_src,src) ;
			return	img_ ;
			}
	static	Xml_E	img_at		(c_tstring& cName,c_tstring& src,c_tstring& name)	{
			Xml_E	img_ = HtmOut::img(cName,src) ;
					img_.AddAttribute(HTM_alt,	name) ;
					img_.AddAttribute(HTM_title,name) ;
			return	img_ ;
			}

	////
	//	*******************************************************************************
	//	<a ...><img ... /></a>
	//	Create	:	2017/02/06
	//	*******************************************************************************
	static	Xml_E	a_img		(c_tstring& cName,c_tstring& src,c_tstring& name)	{
			Xml_E	im ;
			if (!src.empty()) {
					im = HtmOut::img_at(cName,src,name) ;
					}
			Xml_E	ai = HtmOut::a(name,im.ToText()) ;
			return	ai ;
			}

	////
	//	*******************************************************************************
	//	<a ...><br/><a ...><img ... /></a>
	//	Create	:	2017/02/06
	//	*******************************************************************************
	static	Xml_E	a_br_img	(c_tstring& cName,c_tstring& src,c_tstring& name,c_tstring& txt)	{
			Xml_E	a_i ;
			{
				Xml_E	a_ = HtmOut::a				(name,txt) ;
				Xml_E	ai = HtmOut::a_img(cName,src,name) ;
				a_i.AddChild(a_) ;
				a_i.AddChild(HtmOut::br()) ;
				a_i.AddChild(ai) ;
				}
			return	a_i ;
			}

	////
	//	*******************************************************************************
	//	<td><a ...><br/><a ...><img ... /></a></td>
	//	Create	:	2017/02/09
	//	Modify	:	2017/06/27
	//	*******************************************************************************
	static	Xml_E	td_a_br_img	(c_tstring& cName,c_tstring& src,c_tstring& name,c_tstring& txt)	{
			Xml_E	td(HTM_td) ;	//	td.AddAttribute(HTM_align,HTM_center) ;
										td.AddAttribute(HTM_style,CSS_text_align_center()) ;
			{
				Xml_E	ai = HtmOut::a_br_img(cName,src,name,txt) ;
				td.AddChild(ai) ;
				}
			return	td ;
			}

	////
	//	*******************************************************************************
	//	<a ...><img ... /></a>	<a ...>		for debug
	//	Create	:	2017/02/10
	//	*******************************************************************************
	static	Xml_E	img____a	(c_tstring& cName,c_tstring& src,c_tstring& name,c_tstring& txt)	{
			Xml_E	i_a ;
			{
				Xml_E	___i ;
				Xml_E	___a ;
				{
					Xml_E	a_ = HtmOut::a	(name,txt) ;
					Xml_E	ai ;
					if (src.empty())	{	___i.SetText(_T(" ")) ;					}
					else				{	ai = HtmOut::a_img(cName,src,name) ;	}
					___a.AddChild(a_) ;
					___i.AddChild(ai) ;
					}
				i_a.AddChild(___i) ;
				i_a.AddChild(___a) ;
				}
			return	i_a ;
			}

	////
	//	*******************************************************************************
	//	<td><a ...><img ... /></a></td>	<td><a ...></td>
	//	Create	:	2017/02/06
	//	Modify	:	2017/06/27
	//	*******************************************************************************
	static	Xml_E	td_img_td_a	(c_tstring& cName,c_tstring& src,c_tstring& name,c_tstring& txt)	{
			Xml_E	i_a ;
			{
				Xml_E	td_i(HTM_td) ;	//	td_i.AddAttribute(HTM_align,HTM_center) ;
											td_i.AddAttribute(HTM_style,CSS_text_align_center()) ;
				Xml_E	td_a(HTM_td) ;
				{
					Xml_E	a_ = HtmOut::a	(name,txt) ;
					Xml_E	ai ;
					if (src.empty())	{	td_i.SetText(_T(" ")) ;					}
					else				{	ai = HtmOut::a_img(cName,src,name) ;	}
					td_a.AddChild(a_) ;
					td_i.AddChild(ai) ;
					}
				i_a.AddChild(td_i) ;
				i_a.AddChild(td_a) ;
				}
			return	i_a ;
			}

	////
	//	*******************************************************************************
	//	html
	//	Create	:	2015/10/16
	//	Modify	:	2017/01/20
	//	*******************************************************************************
	static	bool	Export		(const Xml_E& elements,LPCTSTR htmName)	{
			v_tstring	strAry ;
			{
				tstring	doc_html = _T("<!DOCTYPE html>") ;
				strAry.push_back(doc_html) ;
				}
			{
				v_tstring	sa = Xml_Out::CreateElement(elements,0) ;
				strAry.insert(strAry.end(),sa.begin(),sa.end()) ;
				}
			tstring	tStr = ::String_Join_Line(strAry) ;
			bool	result = false ;
			{
					result = ::SaveUTF8(htmName,tStr,false) ;
				if(!result) {
					tstring	docName = ::String_ChangePathSP(htmName) ;
					tstring	dmpName = ::Path_AddLastSP(::Get_i_Tools_tmp()) + docName ;
							 ::SaveUTF8(dmpName.c_str(),tStr,false) ;
					}
				}
			return	result ;
			}

	} ;

	/*
	////
	//	*******************************************************************************
	//	img
	//	Create	:	2016/11/18
	//	Modify	:	2017/01/26	Add	img_b ...
	//	Modify	:	2017/02/06	Add	img class ...
	//	*******************************************************************************
	static	Xml_E	img			(c_tstring& src)	{
			Xml_E	img_(HTM_img) ;
					img_.AddAttribute(HTM_src,src) ;
			return	img_ ;
			}

	static	Xml_E	img_b		(c_tstring& src)	{
			Xml_E	img = HtmOut::img(_T(""),src) ;
					img.AddAttribute(HTM_border,_T("1")) ;
			return	img ;
			}
	static	Xml_E	img_b_at_w	(c_tstring& src,c_tstring& name,c_tstring& width)	{
			Xml_E	img = HtmOut::img_b(src) ;
					img.AddAttribute(HTM_alt,	name) ;
					img.AddAttribute(HTM_title,	name) ;
			if (!width.empty()) {
					img.AddAttribute(HTM_width,	width) ;
					}
			return	img ;
			}

	////
	//	*******************************************************************************
	//	<a ...><img ... /></a>
	//	Create	:	2017/02/03
	//	*******************************************************************************
	static	Xml_E	a_img_b		(c_tstring& src,c_tstring& name,c_tstring& width)	{
			Xml_E	im = HtmOut::img_b_at_w(src,name,width) ;
			Xml_E	ai = HtmOut::a(name,im.ToText()) ;
			return	ai ;
			}

	////
	//	*******************************************************************************
	//	<a ...><br/><a ...><img ... /></a>
	//	Create	:	2017/02/03
	//	*******************************************************************************
	static	Xml_E	a_br_img_b	(c_tstring& src,c_tstring& name,c_tstring& width)	{
			Xml_E	a_i ;
			{
				Xml_E	a_ = HtmOut::a			(name) ;
				Xml_E	ai = HtmOut::a_img_b(src,name,width) ;
				a_i.AddChild(a_) ;
				a_i.AddChild(HtmOut::br()) ;
				a_i.AddChild(ai) ;
				}
			return	a_i ;
			}
	////
	//	*******************************************************************************
	//	<td><a ...><img ... /></a></td>	<td><a ...></td>
	//	Create	:	2017/02/03
	//	*******************************************************************************
	static	Xml_E	td_img_b_td_a	(c_tstring& src,c_tstring& name,c_tstring& width)	{
			Xml_E	i_a ;
			{
				Xml_E	td_i(HTM_td) ;		td_i.AddAttribute(HTM_align,HTM_center) ;
				Xml_E	td_a(HTM_td) ;
				{
					Xml_E	a_ = HtmOut::a			(name) ;
					Xml_E	ai = HtmOut::a_img_b(src,name,width) ;
					td_a.AddChild(a_) ;
					td_i.AddChild(ai) ;
					}
				i_a.AddChild(td_i) ;
				i_a.AddChild(td_a) ;
				}
			return	i_a ;
			}
	*/
