//	*******************************************************************************
//	Name	:	textfile.hxx
//			:	Text File Access
//	Author	:	I. Nakagawa
//	Create	:	2014/02/10
//	Modify	:	2014/03/20
//			:	2014/05/01
//			:	2014/10/10
//			:	2014/10/31	down TexPath.csv noting
//			:	2015/07/28	move "Open_File"
//			:	2016/04/22	move "To_v_char"
//			:	2020/01/09	support GNU C
//			:	2020/01/21	add ::SaveText(const long cc,...)
//	*******************************************************************************

#pragma		once

#include	"i_vector.hxx"
#include	"tstring.hxx"
#include	"gettickc.hxx"
#include	"_s_func.hxx"
#include	"tstrmbwc.hxx"
#include	"tstrutf8.hxx"
#include	"stringto.hxx"
#include	"existff.hxx"
#include	"log_msg.hxx"
#include	"t_v_char.hxx"
#include	"vchrfile.hxx"

#ifdef		_MSC_VER
#include	"text_msc.hxx"
//tstring 	to_tstring_msc	(c_v_char& vchr) ;
#else
#include	"text_gnc.hxx"
//tstring 	to_tstring_gnuc	(c_v_char& vchr) ;
#endif

////
//	*******************************************************************************
//	get char code
//	Create	:	2020/01/09
//	*******************************************************************************
inline	long	get_char_code	(LPCTSTR file_name)
{
	v_char	vchr = ::v_c_Load(file_name) ;
	return	::get_char_code(vchr) ;
	}

////
//	*******************************************************************************
//	vectro<char>	->	tstring
//	Create	:	2014/02/13
//	Modify	:	2014/10/10	Add support xml UTF-8
//	Modify	:	2020/01/08	support GNU C
//	*******************************************************************************
inline	tstring	To_tstring	(const v_char& v_chr)
{
	tstring	tStr ;
	#ifdef	_MSC_VER
			tStr = ::to_tstring_msc (v_chr) ;
	#else
			tStr = ::to_tstring_gnuc(v_chr) ;
	#endif
	return	tStr ;
	}

////
//	*******************************************************************************
//	Load Text	tstring
//	Create	:	2014/05/07
//	Modify	:	2014/06/19	Add "File_IsNothing"
//	Modify	:	2020/01/08	support GNU C
//	*******************************************************************************
inline	tstring	LoadText	(LPCTSTR pathName)
{
	if (::File_IsNothing(pathName))	{	return	_T("") ;	}
	v_char	v_chr = ::v_c_Load(pathName) ;
	tstring	tStr = ::To_tstring(v_chr) ;
	return	tStr ;
	}

////
//	*******************************************************************************
//	Save Text	tstring
//	Create	:	2014/05/07
//	*******************************************************************************
inline	bool	SaveText	(LPCTSTR path_name,const tstring& tstr_)
{
	tstring	tstr = tstr_ ;
	#ifdef	_UNICODE
			tstr = ::WString_Add_BOM(tstr) ;
	#endif
	v_char	v_chr = ::To_v_char(tstr) ;
	return	::v_c_Save(path_name,v_chr) ;
	}

////
//	*******************************************************************************
//	Load and Save	vector<tstring>
//	Create	:	2014/05/01
//	Modify	:	2014/05/07
//	Modify	:	2020/01/24
//	*******************************************************************************
inline	bool	LoadText	(LPCTSTR pathName,v_tstring* vtstr)
{
	tstring		tStr = ::LoadText(pathName) ;
	v_tstring	v_tstr = ::String_SplitLine(tStr.c_str()) ;
	*vtstr = v_tstr ;
	return	true ;
	}
inline	bool	SaveText	(LPCTSTR pathName,const v_tstring& vtstr)
{
	tstring	tstr ;
	if (vtstr.size() > 0) {	//	2020/01/24
			tstr = ::String_Join_Line(vtstr) ;
		}
	return	::SaveText(pathName,tstr) ;
	}

inline	v_tstring	LoadTextFile	(LPCTSTR pathName)
{
	v_tstring	vtstr ;
	::LoadText(pathName,&vtstr) ;
	return	vtstr ;
	}
inline	bool		SaveTextFile	(LPCTSTR pathName,const v_tstring& vtstr)
{
	return	::SaveText(pathName,vtstr) ;
	}

////
//	*******************************************************************************
//	Save UTF8	tstring
//	Create	:	2014/05/07
//	Modify	:	2014/05/08		Add parameter bom
//	*******************************************************************************
inline	bool	SaveUTF8	(LPCTSTR pathName,const tstring& tStr,const bool with_BOM=false)
{
	std::string	u8Str = ::To_UTF8(tStr.c_str()) ;
	v_char		v_chr = ::To_v_char(u8Str) ;
	if (with_BOM)	{
		#define	UTF8_BOM	("\xef\xbb\xbf")
		v_char	bom = ::To_v_char(UTF8_BOM) ;
		v_chr.insert(v_chr.begin(),bom.begin(),bom.end()) ;
		}
	return	::v_c_Save    (pathName,v_chr) ;
	}

////
//	*******************************************************************************
//	Save UTF16
//	Create	:	2015/11/02
//	Modify	:	2020/01/21		with_BOM : false -> true
//	*******************************************************************************
inline	bool	SaveUTF16	(LPCTSTR path_name,const tstring& tstr,const bool with_BOM=true)
{
	std::wstring	wcStr = ::To_wstring(tstr.c_str()) ;
	v_char			v_chr = ::To_v_char(wcStr) ;
	if (with_BOM)	{
		#define	UTF16_BOM	("\xff\xfe")
		v_char	bom = ::To_v_char(UTF16_BOM) ;
		v_chr.insert(v_chr.begin(),bom.begin(),bom.end()) ;
		}
	return	::v_c_Save    (path_name,v_chr) ;
	}

////
#ifdef	_MSC_VER
//	*******************************************************************************
//	Save ShiftJIS
//	Create	:	2020/01/21
//	*******************************************************************************
inline	bool	SaveS_JIS	(LPCTSTR pathName,const tstring& tStr)
{
	std::string		mbStr = ::To__string(tStr.c_str()) ;
	v_char			v_chr = ::To_v_char(mbStr) ;
	return	::v_c_Save    (pathName,v_chr) ;
	}

//	*******************************************************************************
//	Save MBCS
//	Create	:	2015/11/02
//	*******************************************************************************
inline	bool	SaveMBCS	(LPCTSTR pathName,const tstring& tStr)
{
	return	::SaveS_JIS	(pathName,tStr) ;
	}
#endif

////
//	*******************************************************************************
//	Save Text
//	Create	:	2020/01/21
//	*******************************************************************************
inline	bool	SaveText	(const long cc,LPCTSTR path_name,const tstring& tstr)
{
	switch	(cc)	{
	#ifdef	_MSC_VER
		case	CC_Shift_JIS	:	return	::SaveS_JIS	(path_name,tstr) ;		break ;
	#endif
		case	CC_UTF_8		:	return	::SaveUTF8	(path_name,tstr) ;		break ;
		case	CC_UTF_16		:	return	::SaveUTF16	(path_name,tstr) ;		break ;
		default					:	return	::SaveText 	(path_name,tstr) ;		break ;
		}
	return	false ;
	}

////
//	*******************************************************************************
//	Save UTF8	tstring array
//	Create	:	2016/04/22
//	*******************************************************************************
inline	bool	SaveUTF8	(LPCTSTR pathName,const v_tstring& strAry,const bool with_BOM=false)
{
	FILE*	ofp = ::Open_File(pathName,_T("wb")) ;
	if (ofp == NULL)	{
		::Log_Message((tstring(_T("Can't open wb  "))+pathName).c_str()) ;
		return	false ;
		}
	if (strAry.size() > 0) {
		if (with_BOM) {
			#define	UTF8_BOM	("\xef\xbb\xbf")
			v_char	bom = ::To_v_char(UTF8_BOM) ;
			::fwrite(&bom[0],sizeof(char),bom.size(),ofp) ;
			}
		#ifdef		MessageBar
		MessageBar	bar(_T("::SaveUTF8 ..."),strAry.size()) ;
		#endif
		for (size_t index=0 ; index<strAry.size() ; index++) {
		#ifdef		MessageBar
			bar.SetBarInc() ;
		#endif
			tstring	str = strAry[index] + _T("\r\n") ;
			{
				std::string	u8Str = ::To_UTF8(str.c_str()) ;
				v_char	v_chr = ::To_v_char(u8Str) ;
				::fwrite(&v_chr[0],sizeof(char),v_chr.size(),ofp) ;
				}
			}
		}
	::fclose(ofp) ;
	return	true ;
	}

